@@ -32,7 +32,7 @@
  */
 
 #define CATCH_CONFIG_MAIN
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 
 #include <cassert>
 #include <iostream>
