@@ -31,7 +31,7 @@
  */
 
 #include "helpers.h"
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "serialization_wrappers.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
@@ -546,6 +546,93 @@ void create_array(
   tiledb_ctx_free(&ctx_array);
 }
 
+tiledb_array_schema_t* create_array_schema(
+    tiledb_ctx_t* ctx,
+    tiledb_array_type_t array_type,
+    const std::vector<std::string>& dim_names,
+    const std::vector<tiledb_datatype_t>& dim_types,
+    const std::vector<void*>& dim_domains,
+    const std::vector<void*>& tile_extents,
+    const std::vector<std::string>& attr_names,
+    const std::vector<tiledb_datatype_t>& attr_types,
+    const std::vector<uint32_t>& cell_val_num,
+    const std::vector<std::pair<tiledb_filter_type_t, int>>& compressors,
+    tiledb_layout_t tile_order,
+    tiledb_layout_t cell_order,
+    uint64_t capacity,
+    bool allows_dups) {
+  // For easy reference
+  auto dim_num = dim_names.size();
+  auto attr_num = attr_names.size();
+
+  // Sanity checks
+  assert(dim_types.size() == dim_num);
+  assert(dim_domains.size() == dim_num);
+  assert(tile_extents.size() == dim_num);
+  assert(attr_types.size() == attr_num);
+  assert(cell_val_num.size() == attr_num);
+  assert(compressors.size() == attr_num);
+
+  // Create array schema
+  tiledb_array_schema_t* array_schema;
+  int rc = tiledb_array_schema_alloc(ctx, array_type, &array_schema);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_set_cell_order(ctx, array_schema, cell_order);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_set_tile_order(ctx, array_schema, tile_order);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_set_capacity(ctx, array_schema, capacity);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_set_allows_dups(ctx, array_schema, (int)allows_dups);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Create dimensions and domain
+  tiledb_domain_t* domain;
+  rc = tiledb_domain_alloc(ctx, &domain);
+  REQUIRE(rc == TILEDB_OK);
+  for (size_t i = 0; i < dim_num; ++i) {
+    tiledb_dimension_t* d;
+    rc = tiledb_dimension_alloc(
+        ctx,
+        dim_names[i].c_str(),
+        dim_types[i],
+        dim_domains[i],
+        tile_extents[i],
+        &d);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_domain_add_dimension(ctx, domain, d);
+    REQUIRE(rc == TILEDB_OK);
+    tiledb_dimension_free(&d);
+  }
+
+  // Set domain to schema
+  rc = tiledb_array_schema_set_domain(ctx, array_schema, domain);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_domain_free(&domain);
+
+  // Create attributes
+  for (size_t i = 0; i < attr_num; ++i) {
+    tiledb_attribute_t* a;
+    rc = tiledb_attribute_alloc(ctx, attr_names[i].c_str(), attr_types[i], &a);
+    REQUIRE(rc == TILEDB_OK);
+    rc = set_attribute_compression_filter(
+        ctx, a, compressors[i].first, compressors[i].second);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_attribute_set_cell_val_num(ctx, a, cell_val_num[i]);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_array_schema_add_attribute(ctx, array_schema, a);
+    REQUIRE(rc == TILEDB_OK);
+    tiledb_attribute_free(&a);
+  }
+
+  // Check array schema
+  rc = tiledb_array_schema_check(ctx, array_schema);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Clean up
+  return array_schema;
+}
+
 void create_s3_bucket(
     const std::string& bucket_name,
     bool s3_supported,
@@ -1216,6 +1303,224 @@ void check_counts(span<T> vals, std::vector<uint64_t> expected) {
     CHECK(counts[i] == expected[i]);
   }
 }
+void serialize_query(
+    const Context& ctx,
+    Query& query,
+    std::vector<uint8_t>* serialized,
+    bool clientside) {
+  ctx.handle_error(serialize_query(
+      ctx.ptr().get(), query.ptr().get(), serialized, clientside));
+}
+
+int serialize_query(
+    tiledb_ctx_t* ctx,
+    tiledb_query_t* query,
+    std::vector<uint8_t>* serialized,
+    bool clientside) {
+  // Serialize
+  tiledb_buffer_list_t* buff_list;
+
+  if (tiledb_serialize_query(
+          ctx, query, TILEDB_CAPNP, clientside ? 1 : 0, &buff_list) !=
+      TILEDB_OK) {
+    return TILEDB_ERR;
+  }
+
+  // Flatten
+  tiledb_buffer_t* c_buff;
+  if (tiledb_buffer_list_flatten(ctx, buff_list, &c_buff) != TILEDB_OK) {
+    return TILEDB_ERR;
+  }
+
+  // Wrap in a safe pointer
+  auto deleter = [](tiledb_buffer_t* b) { tiledb_buffer_free(&b); };
+  std::unique_ptr<tiledb_buffer_t, decltype(deleter)> buff_ptr(c_buff, deleter);
+
+  // Copy into user vector
+  void* data;
+  uint64_t num_bytes;
+  if (tiledb_buffer_get_data(ctx, c_buff, &data, &num_bytes) != TILEDB_OK) {
+    return TILEDB_ERR;
+  }
+  serialized->clear();
+  serialized->insert(
+      serialized->end(),
+      static_cast<const uint8_t*>(data),
+      static_cast<const uint8_t*>(data) + num_bytes);
+
+  // Free buffer list
+  tiledb_buffer_list_free(&buff_list);
+
+  return TILEDB_OK;
+}
+
+void deserialize_query(
+    const Context& ctx,
+    std::vector<uint8_t>& serialized,
+    Query* query,
+    bool clientside) {
+  ctx.handle_error(deserialize_query(
+      ctx.ptr().get(), serialized, query->ptr().get(), clientside));
+}
+
+int deserialize_query(
+    tiledb_ctx_t* ctx,
+    std::vector<uint8_t>& serialized,
+    tiledb_query_t* query,
+    bool clientside) {
+  tiledb_buffer_t* c_buff;
+  if (tiledb_buffer_alloc(ctx, &c_buff) != TILEDB_OK) {
+    return TILEDB_ERR;
+  }
+
+  // Wrap in a safe pointer
+  auto deleter = [](tiledb_buffer_t* b) { tiledb_buffer_free(&b); };
+  std::unique_ptr<tiledb_buffer_t, decltype(deleter)> buff_ptr(c_buff, deleter);
+
+  if (tiledb_buffer_set_data(
+          ctx,
+          c_buff,
+          reinterpret_cast<void*>(&serialized[0]),
+          static_cast<uint64_t>(serialized.size())) != TILEDB_OK) {
+    return TILEDB_ERR;
+  }
+
+  // Deserialize
+  return tiledb_deserialize_query(
+      ctx, c_buff, TILEDB_CAPNP, clientside ? 1 : 0, query);
+}
+
+std::vector<void*> allocate_query_buffers(
+    const Context& ctx, const Array& array, Query* query) {
+  (void)array;
+  std::vector<void*> to_free;
+  void* unused1;
+  uint64_t* unused2;
+  uint8_t* unused3;
+  uint64_t *a1_size, *a2_size, *a2_validity_size, *a3_size, *a3_offset_size,
+      *coords_size;
+  ctx.handle_error(tiledb_query_get_data_buffer(
+      ctx.ptr().get(), query->ptr().get(), "a1", &unused1, &a1_size));
+  ctx.handle_error(tiledb_query_get_data_buffer(
+      ctx.ptr().get(), query->ptr().get(), "a2", &unused1, &a2_size));
+  ctx.handle_error(tiledb_query_get_validity_buffer(
+      ctx.ptr().get(), query->ptr().get(), "a2", &unused3, &a2_validity_size));
+  ctx.handle_error(tiledb_query_get_data_buffer(
+      ctx.ptr().get(), query->ptr().get(), "a3", &unused1, &a3_size));
+  ctx.handle_error(tiledb_query_get_offsets_buffer(
+      ctx.ptr().get(), query->ptr().get(), "a3", &unused2, &a3_offset_size));
+  ctx.handle_error(tiledb_query_get_data_buffer(
+      ctx.ptr().get(),
+      query->ptr().get(),
+      TILEDB_COORDS,
+      &unused1,
+      &coords_size));
+
+  if (a1_size != nullptr) {
+    void* buff = std::malloc(*a1_size);
+    ctx.handle_error(tiledb_query_set_data_buffer(
+        ctx.ptr().get(), query->ptr().get(), "a1", buff, a1_size));
+    to_free.push_back(buff);
+  }
+
+  if (a2_size != nullptr) {
+    void* buff = std::malloc(*a2_size);
+    uint8_t* validity = (uint8_t*)std::malloc(*a2_validity_size);
+    ctx.handle_error(tiledb_query_set_data_buffer(
+        ctx.ptr().get(), query->ptr().get(), "a2", buff, a2_size));
+    ctx.handle_error(tiledb_query_set_validity_buffer(
+        ctx.ptr().get(), query->ptr().get(), "a2", validity, a2_validity_size));
+    to_free.push_back(buff);
+    to_free.push_back(validity);
+  }
+
+  if (a3_size != nullptr) {
+    void* buff = std::malloc(*a3_size);
+    uint64_t* offsets = (uint64_t*)std::malloc(*a3_offset_size);
+    ctx.handle_error(tiledb_query_set_data_buffer(
+        ctx.ptr().get(), query->ptr().get(), "a3", buff, a3_size));
+    ctx.handle_error(tiledb_query_set_offsets_buffer(
+        ctx.ptr().get(), query->ptr().get(), "a3", offsets, a3_offset_size));
+    to_free.push_back(buff);
+    to_free.push_back(offsets);
+  }
+
+  if (coords_size != nullptr) {
+    void* buff = std::malloc(*coords_size);
+    ctx.handle_error(tiledb_query_set_data_buffer(
+        ctx.ptr().get(), query->ptr().get(), TILEDB_COORDS, buff, coords_size));
+    to_free.push_back(buff);
+  }
+
+  return to_free;
+}
+
+void submit_serialized_query(const Context& ctx, Query& query) {
+  submit_serialized_query(ctx.ptr().get(), query.ptr().get());
+}
+void finalize_serialized_query(const Context& ctx, Query& query) {
+  finalize_serialized_query(ctx.ptr().get(), query.ptr().get());
+}
+
+void submit_serialized_query(tiledb_ctx_t* ctx, tiledb_query_t* query) {
+  Context context(ctx, false);
+
+  tiledb_query_type_t type;
+  tiledb_query_get_type(ctx, query, &type);
+  auto uri = query->query_->array()->array_uri().to_string();
+  Array copy_array(context, uri, type);
+  Query copy_query(context, copy_array);
+  std::vector<uint8_t> serialized;
+  context.handle_error(serialize_query(ctx, query, &serialized, true));
+  context.handle_error(
+      deserialize_query(ctx, serialized, copy_query.ptr().get(), false));
+  copy_query.submit();
+  context.handle_error(
+      serialize_query(ctx, copy_query.ptr().get(), &serialized, false));
+  context.handle_error(deserialize_query(ctx, serialized, query, true));
+}
+void finalize_serialized_query(tiledb_ctx_t* ctx, tiledb_query_t* query) {
+  Context context(ctx, false);
+
+  tiledb_query_type_t type;
+  tiledb_query_get_type(ctx, query, &type);
+  auto uri = query->query_->array()->array_uri().to_string();
+  Array copy_array(context, uri, type);
+  Query copy_query(context, copy_array);
+  std::vector<uint8_t> serialized;
+  context.handle_error(serialize_query(ctx, query, &serialized, true));
+  context.handle_error(
+      deserialize_query(ctx, serialized, copy_query.ptr().get(), false));
+  copy_query.finalize();
+  context.handle_error(
+      serialize_query(ctx, copy_query.ptr().get(), &serialized, false));
+  context.handle_error(deserialize_query(ctx, serialized, query, true));
+}
+
+void submit_and_finalize_serialized_query(
+    tiledb_ctx_t* ctx, tiledb_query_t* query) {
+  Context context(ctx, false);
+
+  tiledb_query_type_t type;
+  tiledb_query_get_type(ctx, query, &type);
+  auto uri = query->query_->array()->array_uri().to_string();
+  uint64_t timestamp = query->query_->array()->timestamp_end_opened_at();
+  Array copy_array(context, uri, type, timestamp);
+  Query copy_query(context, copy_array);
+  std::vector<uint8_t> serialized;
+  context.handle_error(serialize_query(ctx, query, &serialized, true));
+  context.handle_error(
+      deserialize_query(ctx, serialized, copy_query.ptr().get(), false));
+  copy_query.submit();
+  copy_query.finalize();
+  context.handle_error(
+      serialize_query(ctx, copy_query.ptr().get(), &serialized, false));
+  context.handle_error(deserialize_query(ctx, serialized, query, true));
+}
+
+void submit_and_finalize_serialized_query(const Context& ctx, Query& query) {
+  submit_and_finalize_serialized_query(ctx.ptr().get(), query.ptr().get());
+}
 
 template void check_subarray<int8_t>(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<int8_t>& ranges);
