@@ -31,10 +31,11 @@
  */
 
 #include "test/src/helpers.h"
+#include "test/src/serialization_wrappers.h"
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include <iostream>
 
 using namespace tiledb::test;
@@ -117,6 +118,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  if (serialized_load) {
+    tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment URI
   const char* uri;
   rc = tiledb_fragment_info_get_fragment_uri(ctx, fragment_info, 1, &uri);
@@ -194,6 +220,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // No fragments yet
   uint32_t fragment_num;
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
@@ -212,6 +263,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment num again
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
   CHECK(rc == TILEDB_OK);
@@ -256,6 +321,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment num again
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
   CHECK(rc == TILEDB_OK);
@@ -288,7 +367,7 @@ TEST_CASE(
   uint64_t size;
   rc = tiledb_fragment_info_get_fragment_size(ctx, fragment_info, 1, &size);
   CHECK(rc == TILEDB_OK);
-  CHECK(size == 3185);
+  CHECK(size == 3202);
 
   // Get dense / sparse
   int32_t dense;
@@ -444,7 +523,33 @@ TEST_CASE(
   CHECK(rc == TILEDB_OK);
   rc = tiledb_fragment_info_load(ctx_correct_key, fragment_info);
   CHECK(rc == TILEDB_OK);
-  tiledb_config_free(&cfg);
+
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_fragment_info_set_config(ctx, deserialized_fragment_info, cfg);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
 
   // No fragments yet
   uint32_t fragment_num;
@@ -473,6 +578,22 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx_correct_key, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_fragment_info_set_config(ctx, deserialized_fragment_info, cfg);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment num again
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
   CHECK(rc == TILEDB_OK);
@@ -517,7 +638,22 @@ TEST_CASE(
   // Load fragment info again
   rc = tiledb_fragment_info_load(ctx_correct_key, fragment_info);
   CHECK(rc == TILEDB_OK);
-  tiledb_ctx_free(&ctx_correct_key);
+
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_fragment_info_set_config(ctx, deserialized_fragment_info, cfg);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
 
   // Get fragment num again
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
@@ -539,7 +675,7 @@ TEST_CASE(
   uint64_t size;
   rc = tiledb_fragment_info_get_fragment_size(ctx, fragment_info, 1, &size);
   CHECK(rc == TILEDB_OK);
-  CHECK(size == 5568);
+  CHECK(size == 5585);
 
   // Get dense / sparse
   int32_t dense;
@@ -746,17 +882,49 @@ TEST_CASE("C API: Test MBR fragment info", "[capi][fragment_info][mbr]") {
   rc = tiledb_fragment_info_alloc(ctx, array_name.c_str(), &fragment_info);
   CHECK(rc == TILEDB_OK);
 
-  // Load fragment info
   rc = tiledb_config_set(cfg, "sm.encryption_key", key, &err);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(err == nullptr);
+
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+    // Set the config that forces preloading of MBRs in remote
+    // case instead of loading them lazily.
+    rc = tiledb_config_set(cfg, "sm.fragment_info.preload_mbrs", "true", &err);
+    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(err == nullptr);
+  }
+#endif
+
   REQUIRE(tiledb_ctx_alloc(cfg, &ctx) == TILEDB_OK);
   rc = tiledb_fragment_info_set_config(ctx, fragment_info, cfg);
   CHECK(rc == TILEDB_OK);
+
+  // Load fragment info
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
   tiledb_config_free(&cfg);
 
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment num
   uint32_t fragment_num;
   rc = tiledb_fragment_info_get_fragment_num(ctx, fragment_info, &fragment_num);
@@ -841,15 +1009,58 @@ TEST_CASE(
   std::string written_frag_uri;
   write_array(ctx, array_name, 1, TILEDB_UNORDERED, buffers, &written_frag_uri);
 
+  // Create config and context
+  tiledb_config_t* cfg;
+  tiledb_error_t* err = nullptr;
+  rc = tiledb_config_alloc(&cfg, &err);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(err == nullptr);
+
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+    // Set the config that forces preloading of MBRs in remote
+    // case instead of loading them lazily.
+    rc = tiledb_config_set(cfg, "sm.fragment_info.preload_mbrs", "true", &err);
+    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(err == nullptr);
+  }
+#endif
+
+  REQUIRE(tiledb_ctx_alloc(cfg, &ctx) == TILEDB_OK);
+
   // Create fragment info object
   tiledb_fragment_info_t* fragment_info = nullptr;
   rc = tiledb_fragment_info_alloc(ctx, array_name.c_str(), &fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  // Set the config
+  rc = tiledb_fragment_info_set_config(ctx, fragment_info, cfg);
+  CHECK(rc == TILEDB_OK);
+
   // Load fragment info
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Load non-empty domain var size - error
   uint64_t domain[2];
   rc = tiledb_fragment_info_get_non_empty_domain_from_index(
@@ -1002,6 +1213,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check for consolidated metadata
   int32_t has;
   rc = tiledb_fragment_info_has_consolidated_metadata(
@@ -1041,6 +1277,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check for consolidated metadata
   rc = tiledb_fragment_info_has_consolidated_metadata(
       ctx, fragment_info, 0, &has);
@@ -1069,6 +1319,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check for consolidated metadata
   rc = tiledb_fragment_info_has_consolidated_metadata(
       ctx, fragment_info, 0, &has);
@@ -1161,6 +1425,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get number of fragments to vacuum
   uint32_t to_vacuum_num;
   rc = tiledb_fragment_info_get_to_vacuum_num(
@@ -1191,6 +1480,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get consolidated fragment URI
   const char* uri;
   rc = tiledb_fragment_info_get_fragment_uri(ctx, fragment_info, 0, &uri);
@@ -1221,6 +1524,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get number of fragments to vacuum
   rc = tiledb_fragment_info_get_to_vacuum_num(
       ctx, fragment_info, &to_vacuum_num);
@@ -1321,6 +1638,31 @@ TEST_CASE("C API: Test fragment info, dump", "[capi][fragment_info][dump]") {
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment array schemas
   tiledb_array_schema_t* frag1_array_schema = nullptr;
   tiledb_array_schema_t* frag2_array_schema = nullptr;
@@ -1376,16 +1718,16 @@ TEST_CASE("C API: Test fragment info, dump", "[capi][fragment_info][dump]") {
       "- Unconsolidated metadata num: 3\n" + "- To vacuum num: 0\n" +
       "- Fragment #1:\n" + "  > URI: " + written_frag_uri_1 + "\n" +
       "  > Type: dense\n" + "  > Non-empty domain: [1, 6]\n" +
-      "  > Size: 3185\n" + "  > Cell num: 10\n" +
+      "  > Size: 3202\n" + "  > Cell num: 10\n" +
       "  > Timestamp range: [1, 1]\n" + "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n" + "- Fragment #2:\n" +
       "  > URI: " + written_frag_uri_2 + "\n" + "  > Type: dense\n" +
-      "  > Non-empty domain: [1, 4]\n" + "  > Size: 3134\n" +
+      "  > Non-empty domain: [1, 4]\n" + "  > Size: 3151\n" +
       "  > Cell num: 5\n" + "  > Timestamp range: [2, 2]\n" +
       "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n" + "- Fragment #3:\n" +
       "  > URI: " + written_frag_uri_3 + "\n" + "  > Type: dense\n" +
-      "  > Non-empty domain: [5, 6]\n" + "  > Size: 3182\n" +
+      "  > Non-empty domain: [5, 6]\n" + "  > Size: 3202\n" +
       "  > Cell num: 10\n" + "  > Timestamp range: [3, 3]\n" +
       "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n";
@@ -1503,6 +1845,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get consolidated fragment URI
   const char* uri;
   rc = tiledb_fragment_info_get_fragment_uri(ctx, fragment_info, 0, &uri);
@@ -1516,7 +1883,7 @@ TEST_CASE(
       "- To vacuum URIs:\n" + "  > " + written_frag_uri_1 + "\n  > " +
       written_frag_uri_2 + "\n  > " + written_frag_uri_3 + "\n" +
       "- Fragment #1:\n" + "  > URI: " + uri + "\n" + "  > Type: dense\n" +
-      "  > Non-empty domain: [1, 10]\n" + "  > Size: 3200\n" +
+      "  > Non-empty domain: [1, 10]\n" + "  > Size: 3208\n" +
       "  > Cell num: 10\n" + "  > Timestamp range: [1, 3]\n" +
       "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n";
@@ -1593,6 +1960,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check dump
   const auto ver = std::to_string(tiledb::sm::constants::format_version);
   std::string dump_str =
@@ -1600,7 +1992,7 @@ TEST_CASE(
       "- Unconsolidated metadata num: 1\n" + "- To vacuum num: 0\n" +
       "- Fragment #1:\n" + "  > URI: " + written_frag_uri + "\n" +
       "  > Type: sparse\n" + "  > Non-empty domain: [a, ddd]\n" +
-      "  > Size: 3431\n" + "  > Cell num: 4\n" +
+      "  > Size: 3439\n" + "  > Cell num: 4\n" +
       "  > Timestamp range: [1, 1]\n" + "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n";
   FILE* gold_fout = fopen("gold_fout.txt", "w");
@@ -1688,6 +2080,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Get fragment URI
   const char* frag_uri;
   rc = tiledb_fragment_info_get_fragment_uri(ctx, fragment_info, 0, &frag_uri);
@@ -1778,6 +2195,31 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  tiledb_fragment_info_t* deserialized_fragment_info = nullptr;
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check for consolidated metadata
   int32_t has;
   rc = tiledb_fragment_info_has_consolidated_metadata(
@@ -1845,6 +2287,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check for consolidated metadata
   rc = tiledb_fragment_info_has_consolidated_metadata(
       ctx, fragment_info, 0, &has);
@@ -1873,6 +2329,20 @@ TEST_CASE(
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
+  if (serialized_load) {
+    rc = tiledb_fragment_info_alloc(
+        ctx, array_name.c_str(), &deserialized_fragment_info);
+    CHECK(rc == TILEDB_OK);
+    tiledb_fragment_info_serialize(
+        ctx,
+        array_name.c_str(),
+        fragment_info,
+        deserialized_fragment_info,
+        tiledb_serialization_type_t(0));
+    tiledb_fragment_info_free(&fragment_info);
+    fragment_info = deserialized_fragment_info;
+  }
+
   // Check again
   rc = tiledb_fragment_info_has_consolidated_metadata(
       ctx, fragment_info, 1, &has);
