@@ -30,7 +30,7 @@
  * Tests for query serialization/deserialization.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "test/src/helpers.h"
 #include "tiledb/common/common.h"
 #include "tiledb/sm/c_api/tiledb.h"
@@ -54,6 +54,10 @@
 using namespace tiledb;
 using ResultSetType = std::map<std::string, std::any>;
 
+using tiledb::test::allocate_query_buffers;
+using tiledb::test::deserialize_query;
+using tiledb::test::serialize_query;
+
 namespace {
 
 #ifdef _WIN32
@@ -141,11 +145,22 @@ struct SerializationFx {
     REQUIRE(loop_num->second > 0);
   }
 
+  static void check_delete_stats(const Query& query) {
+    auto stats = ((sm::Reader*)query.ptr()->query_->strategy())->stats();
+    REQUIRE(stats != nullptr);
+    auto counters = stats->counters();
+    REQUIRE(counters != nullptr);
+    auto dowork_num = counters->find(
+        "Context.StorageManager.Query.Deletes.dowork.timer_count");
+    REQUIRE((dowork_num != counters->end()));
+    REQUIRE(dowork_num->second > 0);
+  }
+
   void create_array(tiledb_array_type_t type) {
     ArraySchema schema(ctx, type);
     Domain domain(ctx);
-    domain.add_dimension(Dimension::create<int32_t>(ctx, "d1", {1, 10}, 2))
-        .add_dimension(Dimension::create<int32_t>(ctx, "d2", {1, 10}, 2));
+    domain.add_dimension(Dimension::create<int32_t>(ctx, "d1", {0, 100}, 2))
+        .add_dimension(Dimension::create<int32_t>(ctx, "d2", {0, 10}, 2));
     schema.set_domain(domain);
 
     schema.add_attribute(Attribute::create<uint32_t>(ctx, "a1"));
@@ -317,6 +332,33 @@ struct SerializationFx {
     check_write_stats(query);
   }
 
+  void write_sparse_delete() {
+    Array array(ctx, array_uri, TILEDB_DELETE);
+    Query query(ctx, array);
+
+    // Define query condition (a1 < 5).
+    QueryCondition qc(ctx);
+    int32_t val = 5;
+    qc.init("a1", &val, sizeof(int32_t), TILEDB_LT);
+    query.set_condition(qc);
+
+    // Serialize into a copy and submit.
+    std::vector<uint8_t> serialized;
+    serialize_query(ctx, query, &serialized, true);
+    Array array2(ctx, array_uri, TILEDB_DELETE);
+    Query query2(ctx, array2);
+    deserialize_query(ctx, serialized, &query2, false);
+    query2.submit();
+
+    // Make sure query2 has logged stats
+    check_delete_stats(query2);
+    serialize_query(ctx, query2, &serialized, false);
+    deserialize_query(ctx, serialized, &query, true);
+
+    // The deserialized query should also include the delete stats
+    check_delete_stats(query);
+  }
+
   void write_sparse_array_split_coords() {
     std::vector<int32_t> d1 = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
 
@@ -370,173 +412,6 @@ struct SerializationFx {
     // The deserialized query should also include the write stats
     check_write_stats(query);
   }
-
-  /**
-   * Helper function that serializes a query from the "client" or "server"
-   * perspective. The flow being mimicked here is (for read queries):
-   *
-   * - Client sets up read query object including buffers.
-   * - Client submits query to a remote array.
-   * - Internal code (not C API) serializes that query and send it via curl.
-   * - Server receives and deserializes the query using the C API.
-   * - Server submits query.
-   * - Server serializes (using C API) the query and sends it back.
-   * - Client receives response and deserializes the query (not C API). This
-   *   copies the query results into the original user buffers.
-   * - Client's blocking call to tiledb_query_submit() now returns.
-   */
-  static void serialize_query(
-      const Context& ctx,
-      Query& query,
-      std::vector<uint8_t>* serialized,
-      bool clientside) {
-    // Serialize
-    tiledb_buffer_list_t* buff_list;
-
-    ctx.handle_error(tiledb_serialize_query(
-        ctx.ptr().get(),
-        query.ptr().get(),
-        TILEDB_CAPNP,
-        clientside ? 1 : 0,
-        &buff_list));
-
-    // Flatten
-    tiledb_buffer_t* c_buff;
-    ctx.handle_error(
-        tiledb_buffer_list_flatten(ctx.ptr().get(), buff_list, &c_buff));
-
-    // Wrap in a safe pointer
-    auto deleter = [](tiledb_buffer_t* b) { tiledb_buffer_free(&b); };
-    std::unique_ptr<tiledb_buffer_t, decltype(deleter)> buff_ptr(
-        c_buff, deleter);
-
-    // Copy into user vector
-    void* data;
-    uint64_t num_bytes;
-    ctx.handle_error(
-        tiledb_buffer_get_data(ctx.ptr().get(), c_buff, &data, &num_bytes));
-    serialized->clear();
-    serialized->insert(
-        serialized->end(),
-        static_cast<const uint8_t*>(data),
-        static_cast<const uint8_t*>(data) + num_bytes);
-
-    // Free buffer list
-    tiledb_buffer_list_free(&buff_list);
-  }
-
-  /**
-   * Helper function that deserializes a query from the "client" or "server"
-   * perspective.
-   */
-  static void deserialize_query(
-      const Context& ctx,
-      std::vector<uint8_t>& serialized,
-      Query* query,
-      bool clientside) {
-    tiledb_buffer_t* c_buff;
-    ctx.handle_error(tiledb_buffer_alloc(ctx.ptr().get(), &c_buff));
-
-    // Wrap in a safe pointer
-    auto deleter = [](tiledb_buffer_t* b) { tiledb_buffer_free(&b); };
-    std::unique_ptr<tiledb_buffer_t, decltype(deleter)> buff_ptr(
-        c_buff, deleter);
-
-    ctx.handle_error(tiledb_buffer_set_data(
-        ctx.ptr().get(),
-        c_buff,
-        reinterpret_cast<void*>(&serialized[0]),
-        static_cast<uint64_t>(serialized.size())));
-
-    // Deserialize
-    ctx.handle_error(tiledb_deserialize_query(
-        ctx.ptr().get(),
-        c_buff,
-        TILEDB_CAPNP,
-        clientside ? 1 : 0,
-        query->ptr().get()));
-  }
-
-  /**
-   * Helper function that allocates buffers on a query object that has been
-   * deserialized on the "server" side.
-   */
-  static std::vector<void*> allocate_query_buffers(
-      const Context& ctx, const Array& array, Query* query) {
-    (void)array;
-    std::vector<void*> to_free;
-    void* unused1;
-    uint64_t* unused2;
-    uint8_t* unused3;
-    uint64_t *a1_size, *a2_size, *a2_validity_size, *a3_size, *a3_offset_size,
-        *coords_size;
-    ctx.handle_error(tiledb_query_get_data_buffer(
-        ctx.ptr().get(), query->ptr().get(), "a1", &unused1, &a1_size));
-    ctx.handle_error(tiledb_query_get_data_buffer(
-        ctx.ptr().get(), query->ptr().get(), "a2", &unused1, &a2_size));
-    ctx.handle_error(tiledb_query_get_validity_buffer(
-        ctx.ptr().get(),
-        query->ptr().get(),
-        "a2",
-        &unused3,
-        &a2_validity_size));
-    ctx.handle_error(tiledb_query_get_data_buffer(
-        ctx.ptr().get(), query->ptr().get(), "a3", &unused1, &a3_size));
-    ctx.handle_error(tiledb_query_get_offsets_buffer(
-        ctx.ptr().get(), query->ptr().get(), "a3", &unused2, &a3_offset_size));
-    ctx.handle_error(tiledb_query_get_data_buffer(
-        ctx.ptr().get(),
-        query->ptr().get(),
-        TILEDB_COORDS,
-        &unused1,
-        &coords_size));
-
-    if (a1_size != nullptr) {
-      void* buff = std::malloc(*a1_size);
-      ctx.handle_error(tiledb_query_set_data_buffer(
-          ctx.ptr().get(), query->ptr().get(), "a1", buff, a1_size));
-      to_free.push_back(buff);
-    }
-
-    if (a2_size != nullptr) {
-      void* buff = std::malloc(*a2_size);
-      uint8_t* validity = (uint8_t*)std::malloc(*a2_validity_size);
-      ctx.handle_error(tiledb_query_set_data_buffer(
-          ctx.ptr().get(), query->ptr().get(), "a2", buff, a2_size));
-      ctx.handle_error(tiledb_query_set_validity_buffer(
-          ctx.ptr().get(),
-          query->ptr().get(),
-          "a2",
-          validity,
-          a2_validity_size));
-      to_free.push_back(buff);
-      to_free.push_back(validity);
-    }
-
-    if (a3_size != nullptr) {
-      void* buff = std::malloc(*a3_size);
-      uint64_t* offsets = (uint64_t*)std::malloc(*a3_offset_size);
-      ctx.handle_error(tiledb_query_set_data_buffer(
-          ctx.ptr().get(), query->ptr().get(), "a3", buff, a3_size));
-      ctx.handle_error(tiledb_query_set_offsets_buffer(
-          ctx.ptr().get(), query->ptr().get(), "a3", offsets, a3_offset_size));
-      to_free.push_back(buff);
-      to_free.push_back(offsets);
-    }
-
-    if (coords_size != nullptr) {
-      void* buff = std::malloc(*coords_size);
-      ctx.handle_error(tiledb_query_set_data_buffer(
-          ctx.ptr().get(),
-          query->ptr().get(),
-          TILEDB_COORDS,
-          buff,
-          coords_size));
-      to_free.push_back(buff);
-    }
-
-    return to_free;
-  }
 };
 
 }  // namespace
@@ -586,7 +461,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -649,7 +524,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     // We expect all cells where `a1` >= `cmp_value` to be filtered
@@ -728,7 +603,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -796,7 +671,7 @@ TEST_CASE_METHOD(
       // Deserialize into original query (client side).
       deserialize_query(ctx, serialized, &q, true);
 
-      // The deserialized query should also include the write stats
+      // The deserialized query should also include the read stats
       check_read_stats(query);
 
       for (void* b : to_free)
@@ -888,7 +763,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -1015,7 +890,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -1079,7 +954,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -1134,7 +1009,7 @@ TEST_CASE_METHOD(
     deserialize_query(ctx, serialized, &query, true);
     REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
-    // The deserialized query should also include the write stats
+    // The deserialized query should also include the read stats
     check_read_stats(query);
 
     auto result_el = query.result_buffer_elements_nullable();
@@ -1191,7 +1066,7 @@ TEST_CASE_METHOD(
       // Deserialize into original query (client side).
       deserialize_query(ctx, serialized, &q, true);
 
-      // The deserialized query should also include the write stats
+      // The deserialized query should also include the read stats
       check_read_stats(query);
 
       for (void* b : to_free)
@@ -1237,3 +1112,192 @@ TEST_CASE_METHOD(
     // TODO: check results
   }
 }
+
+TEST_CASE_METHOD(
+    SerializationFx,
+    "Query serialization, sparse delete",
+    "[query][sparse][delete][serialization]") {
+  create_array(TILEDB_SPARSE);
+  write_sparse_array();
+  write_sparse_delete();
+
+  SECTION("- Read all") {
+    Array array(ctx, array_uri, TILEDB_READ);
+    Query query(ctx, array);
+    std::vector<int32_t> coords(1000);
+    std::vector<uint32_t> a1(1000);
+    std::vector<uint32_t> a2(1000);
+    std::vector<uint8_t> a2_nullable(1000);
+    std::vector<char> a3_data(1000 * 100);
+    std::vector<uint64_t> a3_offsets(1000);
+    std::vector<int32_t> subarray = {1, 10, 1, 10};
+
+    query.set_subarray(subarray);
+    query.set_coordinates(coords);
+    query.set_data_buffer("a1", a1);
+    query.set_data_buffer("a2", a2);
+    query.set_validity_buffer("a2", a2_nullable);
+    query.set_data_buffer("a3", a3_data);
+    query.set_offsets_buffer("a3", a3_offsets);
+
+    // Serialize into a copy and submit.
+    std::vector<uint8_t> serialized;
+    serialize_query(ctx, query, &serialized, true);
+    Array array2(ctx, array_uri, TILEDB_READ);
+    Query query2(ctx, array2);
+    deserialize_query(ctx, serialized, &query2, false);
+    auto to_free = allocate_query_buffers(ctx, array2, &query2);
+    query2.submit();
+    serialize_query(ctx, query2, &serialized, false);
+
+    // Make sure query2 has logged stats
+    check_read_stats(query2);
+
+    // Deserialize into original query
+    deserialize_query(ctx, serialized, &query, true);
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
+
+    // The deserialized query should also include the read stats
+    check_read_stats(query);
+
+    auto result_el = query.result_buffer_elements_nullable();
+    REQUIRE(std::get<1>(result_el["a1"]) == 5);
+    REQUIRE(std::get<1>(result_el["a2"]) == 10);
+    REQUIRE(std::get<2>(result_el["a2"]) == 5);
+    REQUIRE(std::get<0>(result_el["a3"]) == 5);
+    REQUIRE(std::get<1>(result_el["a3"]) == 40);
+
+    // TODO: check results
+
+    for (void* b : to_free)
+      std::free(b);
+  }
+}
+
+TEST_CASE_METHOD(
+    SerializationFx,
+    "Global order writes serialization",
+    "[global-order-write][serialization][dense]") {
+  uint64_t tile_extent = 2;
+  ArraySchema schema(ctx, TILEDB_DENSE);
+  Domain domain(ctx);
+  domain.add_dimension(
+      Dimension::create<uint64_t>(ctx, "d1", {0, 200}, tile_extent));
+  schema.set_domain(domain);
+  schema.add_attribute(Attribute::create<uint32_t>(ctx, "a1"));
+  schema.add_attribute(
+      Attribute::create<std::array<uint32_t, 2>>(ctx, "a2").set_nullable(true));
+  schema.add_attribute(Attribute::create<std::vector<char>>(ctx, "a3"));
+  Array::create(array_uri, schema);
+
+  // Build input data
+  uint64_t ncells = 100;
+  // This needs to be tile-aligned
+  uint64_t chunk_size = 4;
+
+  std::vector<uint32_t> a1;
+  std::vector<uint32_t> a2;
+  std::vector<uint8_t> a2_nullable;
+  std::vector<char> a3_data;
+  std::vector<uint64_t> a3_offsets;
+  for (uint64_t i = 0; i < ncells; i++) {
+    a1.push_back(i);
+    a2.push_back(i);
+    a2.push_back(2 * i);
+    a2_nullable.push_back(a2.back() % 5 == 0 ? 0 : 1);
+    std::string a3 = "abcd";
+    a3_offsets.push_back(i % chunk_size * a3.size());
+    a3_data.insert(a3_data.end(), a3.begin(), a3.end());
+  }
+
+  Array array(ctx, array_uri, TILEDB_WRITE);
+  Query query(ctx, array);
+  Subarray subarray(ctx, array);
+  query.set_layout(TILEDB_GLOBAL_ORDER);
+
+  uint64_t last_space_tile =
+      (ncells / tile_extent +
+       static_cast<uint64_t>(ncells % tile_extent != 0)) *
+          tile_extent -
+      1;
+  subarray.add_range(0, static_cast<uint64_t>(0), last_space_tile);
+  query.set_subarray(subarray);
+
+  uint64_t begin = 0;
+  uint64_t end = chunk_size - 1;
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  while (begin < end) {
+    query.set_data_buffer("a1", a1.data() + begin, end - begin + 1);
+    query.set_data_buffer("a2", a2.data() + begin * 2, (end - begin + 1) * 2);
+    query.set_validity_buffer(
+        "a2", a2_nullable.data() + begin, end - begin + 1);
+    query.set_data_buffer(
+        "a3", a3_data.data() + begin * 4, (end - begin + 1) * 4);
+    query.set_offsets_buffer("a3", a3_offsets.data() + begin, end - begin + 1);
+    begin += chunk_size;
+    end = std::min(last_space_tile, end + chunk_size);
+
+    // Simulate REST submit()
+    if (begin < end) {
+      if (!serialized_writes) {
+        query.submit();
+      } else {
+        test::submit_serialized_query(ctx, query);
+      }
+    }
+  }
+
+  if (!serialized_writes) {
+    query.submit();
+    query.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx, query);
+  }
+
+  REQUIRE(query.query_status() == Query::Status::COMPLETE);
+
+  // Read and validate results
+  {
+    Array array(ctx, array_uri, TILEDB_READ);
+    Query query(ctx, array);
+    std::vector<uint32_t> a1_result(ncells);
+    std::vector<uint32_t> a2_result(ncells * 2);
+    std::vector<uint8_t> a2_result_nullable(ncells);
+    std::vector<char> a3_result_data(4 * ncells);
+    std::vector<uint64_t> a3_result_offsets(ncells);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, static_cast<uint64_t>(0), ncells - 1);
+    query.set_subarray(subarray);
+    query.set_data_buffer("a1", a1_result.data(), a1_result.size());
+    query.set_data_buffer("a2", a2_result.data(), a2_result.size());
+    query.set_validity_buffer(
+        "a2", a2_result_nullable.data(), a2_result_nullable.size());
+    query.set_data_buffer("a3", a3_result_data.data(), a3_result_data.size());
+    query.set_offsets_buffer(
+        "a3", a3_result_offsets.data(), a3_result_offsets.size());
+
+    query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
+
+    for (uint64_t i = 0; i < ncells; ++i) {
+      CHECK(a1[i] == a1_result[i]);
+    }
+    for (uint64_t i = 0; i < ncells * 2; ++i) {
+      CHECK(a2[i] == a2_result[i]);
+    }
+    for (uint64_t i = 0; i < ncells; ++i) {
+      CHECK(a2_nullable[i] == a2_result_nullable[i]);
+    }
+    for (uint64_t i = 0; i < ncells * 4; ++i) {
+      CHECK(a3_data[i] == a3_result_data[i]);
+    }
+  }
+}
