@@ -33,6 +33,7 @@
 #ifndef TILEDB_VFS_HELPERS_H
 #define TILEDB_VFS_HELPERS_H
 
+#include "catch.hpp"
 #include "test/src/helpers.h"
 
 #ifdef _WIN32
@@ -541,6 +542,91 @@ class SupportedFsMem : public SupportedFs {
   std::string temp_dir_;
 };
 
+/**
+ * Fixture for creating a temporary directory for a test case. This fixture
+ * also manages the context and virtual file system for the test case.
+ */
+struct TemporaryDirectoryFixture {
+ public:
+  /** Fixture constructor. */
+  TemporaryDirectoryFixture()
+      : supported_filesystems_(vfs_test_get_fs_vec()) {
+    // Initialize virtual filesystem and context.
+    REQUIRE(vfs_test_init(supported_filesystems_, &ctx, &vfs_).ok());
+
+    // Create temporary directory based on the supported filesystem
+#ifdef _WIN32
+    SupportedFsLocal windows_fs;
+    temp_dir_ = windows_fs.file_prefix() + windows_fs.temp_dir();
+#else
+    SupportedFsLocal posix_fs;
+    temp_dir_ = posix_fs.file_prefix() + posix_fs.temp_dir();
+#endif
+    create_dir(temp_dir_, ctx, vfs_);
+  }
+
+  /** Fixture destructor. */
+  ~TemporaryDirectoryFixture() {
+    remove_dir(temp_dir_, ctx, vfs_);
+    tiledb_ctx_free(&ctx);
+    tiledb_vfs_free(&vfs_);
+  }
+
+  /** Create a path in the temporary directory. */
+  std::string fullpath(std::string&& name) {
+    return temp_dir_ + name;
+  }
+
+  /**
+   * Creates a new array array in the temporary directory and returns the
+   * fullpath the array.
+   *
+   * @param name Name of the array relative to the temporary directory.
+   * @param array_schema Schema for the array to be created.
+   * @returns URI of the array.
+   */
+  std::string create_temporary_array(
+      std::string&& name, tiledb_array_schema_t* array_schema) {
+    auto array_uri = fullpath(std::move(name));
+    REQUIRE_TILEDB_OK(tiledb_array_schema_check(ctx, array_schema));
+    REQUIRE_TILEDB_OK(
+        tiledb_array_create(ctx, array_uri.c_str(), array_schema));
+    return array_uri;
+  };
+
+  /**
+   * Checks the return code for a TileDB C-API function is TILEDB_OK. If not,
+   * if will add a failed assert to the Catch2 test and print the last error
+   * message from the local TileDB context.
+   *
+   * @param rc Return code from a TileDB C-API function.
+   */
+  void check_tiledb_ok(int rc);
+
+  /**
+   * Requires the return code for a TileDB C-API function is TILEDB_OK. If not,
+   * it will end the Catch2 test and print the last error message from the local
+   * TileDB context.
+   *
+   * @param rc Return code from a TileDB C-API function.
+   */
+  void require_tiledb_ok(int rc);
+
+ protected:
+  /** TileDB context */
+  tiledb_ctx_t* ctx;
+
+  /** Name of the temporary directory to use for this test */
+  std::string temp_dir_;
+
+ private:
+  /** Virtual file system */
+  tiledb_vfs_t* vfs_;
+
+  /** Vector of supported filesystems. Used to initialize ``vfs_``. */
+  const std::vector<std::unique_ptr<SupportedFs>> supported_filesystems_;
+};
+
 }  // End of namespace test
 }  // End of namespace tiledb
 
