@@ -84,6 +84,10 @@ const URI& ArrayDirectory::latest_array_schema_uri() const {
   return latest_array_schema_uri_;
 }
 
+const std::vector<URI>& ArrayDirectory::unfiltered_fragment_uris() const {
+  return unfiltered_fragment_uris_;
+}
+
 const std::vector<URI>& ArrayDirectory::array_meta_uris_to_vacuum() const {
   return array_meta_uris_to_vacuum_;
 }
@@ -100,6 +104,11 @@ const std::vector<URI>& ArrayDirectory::commit_uris_to_vacuum() const {
   return commit_uris_to_vacuum_;
 }
 
+const std::unordered_set<std::string>&
+ArrayDirectory::consolidated_commit_uris_set() const {
+  return consolidated_commit_uris_set_;
+}
+
 const std::vector<URI>& ArrayDirectory::consolidated_commits_uris_to_vacuum()
     const {
   return consolidated_commits_uris_to_vacuum_;
@@ -235,6 +244,11 @@ Status ArrayDirectory::load() {
           fragment_meta_uris_v12_or_higher.begin(),
           fragment_meta_uris_v12_or_higher.end(),
           std::back_inserter(fragment_meta_uris_));
+
+      // Sort the delete commits by timestamp. Delete tiles locations come from
+      // both the consolidated file and the directory listing, and they might
+      // have interleaved times, so we need to sort.
+      std::sort(delete_tiles_location_.begin(), delete_tiles_location_.end());
     }
   }
 
@@ -259,16 +273,11 @@ ArrayDirectory::filtered_fragment_uris(const bool full_overlap_only) const {
   std::vector<URI> commit_uris_to_ignore;
   if (mode_ == ArrayDirectoryMode::VACUUM_FRAGMENTS) {
     for (auto& uri : fragment_uris_to_vacuum.value()) {
-      auto&& [st, commit_uri] = get_commit_uri(uri);
-      if (!st.ok()) {
-        throw std::logic_error(st.message());
-      }
-
-      if (consolidated_commit_uris_set_.count(commit_uri.value().c_str()) ==
-          0) {
-        commit_uris_to_vacuum.emplace_back(commit_uri.value());
+      auto commit_uri = get_commit_uri(uri);
+      if (consolidated_commit_uris_set_.count(commit_uri.c_str()) == 0) {
+        commit_uris_to_vacuum.emplace_back(commit_uri);
       } else {
-        commit_uris_to_ignore.emplace_back(commit_uri.value());
+        commit_uris_to_ignore.emplace_back(commit_uri);
       }
     }
   }
@@ -299,12 +308,17 @@ ArrayDirectory::delete_tiles_location() const {
   return delete_tiles_location_;
 }
 
-URI ArrayDirectory::get_fragments_dir(uint32_t write_version) const {
+URI ArrayDirectory::generate_fragment_dir_uri(
+    uint32_t write_version, URI array_uri) {
   if (write_version < 12) {
-    return uri_;
+    return array_uri;
   }
 
-  return uri_.join_path(constants::array_fragments_dir_name);
+  return array_uri.join_path(constants::array_fragments_dir_name);
+}
+
+URI ArrayDirectory::get_fragments_dir(uint32_t write_version) const {
+  return generate_fragment_dir_uri(write_version, uri_);
 }
 
 URI ArrayDirectory::get_fragment_metadata_dir(uint32_t write_version) const {
@@ -323,40 +337,32 @@ URI ArrayDirectory::get_commits_dir(uint32_t write_version) const {
   return uri_.join_path(constants::array_commits_dir_name);
 }
 
-tuple<Status, optional<URI>> ArrayDirectory::get_commit_uri(
-    const URI& fragment_uri) const {
+URI ArrayDirectory::get_commit_uri(const URI& fragment_uri) const {
   auto name = fragment_uri.remove_trailing_slash().last_path_part();
   uint32_t version;
-  RETURN_NOT_OK_TUPLE(
-      utils::parse::get_fragment_version(name, &version), nullopt);
+  throw_if_not_ok(utils::parse::get_fragment_version(name, &version));
 
   if (version == UINT32_MAX || version < 12) {
-    return {Status::Ok(),
-            URI(fragment_uri.to_string() + constants::ok_file_suffix)};
+    return URI(fragment_uri.to_string() + constants::ok_file_suffix);
   }
 
   auto temp_uri =
       uri_.join_path(constants::array_commits_dir_name).join_path(name);
-  return {Status::Ok(),
-          URI(temp_uri.to_string() + constants::write_file_suffix)};
+  return URI(temp_uri.to_string() + constants::write_file_suffix);
 }
 
-tuple<Status, optional<URI>> ArrayDirectory::get_vaccum_uri(
-    const URI& fragment_uri) const {
+URI ArrayDirectory::get_vacuum_uri(const URI& fragment_uri) const {
   auto name = fragment_uri.remove_trailing_slash().last_path_part();
   uint32_t version;
-  RETURN_NOT_OK_TUPLE(
-      utils::parse::get_fragment_version(name, &version), nullopt);
+  throw_if_not_ok(utils::parse::get_fragment_version(name, &version));
 
   if (version == UINT32_MAX || version < 12) {
-    return {Status::Ok(),
-            URI(fragment_uri.to_string() + constants::vacuum_file_suffix)};
+    return URI(fragment_uri.to_string() + constants::vacuum_file_suffix);
   }
 
   auto temp_uri =
       uri_.join_path(constants::array_commits_dir_name).join_path(name);
-  return {Status::Ok(),
-          URI(temp_uri.to_string() + constants::vacuum_file_suffix)};
+  return URI(temp_uri.to_string() + constants::vacuum_file_suffix);
 }
 
 tuple<Status, optional<std::string>> ArrayDirectory::compute_new_fragment_name(
@@ -454,9 +460,23 @@ ArrayDirectory::load_commits_dir_uris_v12_or_higher(
     } else if (stdx::string::ends_with(
                    commits_dir_uris[i].to_string(),
                    constants::delete_file_suffix)) {
-      if (consolidated_commit_uris_set_.count(commits_dir_uris[i].c_str()) ==
-          0) {
-        delete_tiles_location_.emplace_back(commits_dir_uris[i], 0);
+      // Get the start and end timestamp for this delete
+      std::pair<uint64_t, uint64_t> delete_timestamp_range;
+      RETURN_NOT_OK_TUPLE(
+          utils::parse::get_timestamp_range(
+              commits_dir_uris[i], &delete_timestamp_range),
+          nullopt);
+
+      // Add the delete tile location if it overlaps the open start/end times
+      if (timestamps_overlap(delete_timestamp_range, false)) {
+        if (consolidated_commit_uris_set_.count(commits_dir_uris[i].c_str()) ==
+            0) {
+          const auto base_uri_size = uri_.to_string().size();
+          delete_tiles_location_.emplace_back(
+              commits_dir_uris[i],
+              commits_dir_uris[i].to_string().substr(base_uri_size),
+              0);
+        }
       }
     }
   }
@@ -516,9 +536,35 @@ ArrayDirectory::load_consolidated_commit_uris(
       RETURN_NOT_OK_TUPLE(
           vfs_->read(uri, 0, &names[0], size), nullopt, nullopt);
       std::stringstream ss(names);
-      for (std::string uri_str; std::getline(ss, uri_str);) {
-        if (ignore_set.count(uri_str) == 0) {
-          uris_set.emplace(uri_.to_string() + uri_str);
+      for (std::string condition_marker; std::getline(ss, condition_marker);) {
+        if (ignore_set.count(condition_marker) == 0) {
+          uris_set.emplace(uri_.to_string() + condition_marker);
+        }
+
+        // If we have a delete, process the condition tile
+        if (stdx::string::ends_with(
+                condition_marker, constants::delete_file_suffix)) {
+          storage_size_t size = 0;
+          ss.read(
+              static_cast<char*>(static_cast<void*>(&size)),
+              sizeof(storage_size_t));
+          auto pos = ss.tellg();
+
+          // Get the start and end timestamp for this delete
+          std::pair<uint64_t, uint64_t> delete_timestamp_range;
+          RETURN_NOT_OK_TUPLE(
+              utils::parse::get_timestamp_range(
+                  URI(condition_marker), &delete_timestamp_range),
+              nullopt,
+              nullopt);
+
+          // Add the delete tile location if it overlaps the open start/end
+          // times
+          if (timestamps_overlap(delete_timestamp_range, false)) {
+            delete_tiles_location_.emplace_back(uri, condition_marker, pos);
+          }
+          pos += size;
+          ss.seekg(pos);
         }
       }
     }
@@ -635,7 +681,9 @@ void ArrayDirectory::load_commits_uris_to_consolidate(
   // Add the wrt file URIs not already in the list.
   for (auto& uri : commits_dir_uris) {
     if (stdx::string::ends_with(
-            uri.to_string(), constants::write_file_suffix)) {
+            uri.to_string(), constants::write_file_suffix) ||
+        stdx::string::ends_with(
+            uri.to_string(), constants::delete_file_suffix)) {
       if (consolidated_uris_set.count(uri.c_str()) == 0) {
         commit_uris_to_consolidate_.emplace_back(uri);
       }
