@@ -30,7 +30,7 @@
  * Tests for the dictionary encoding of strings.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 
 #include "../dict_compressor.h"
 
@@ -274,6 +274,7 @@ TEMPLATE_LIST_TEST_CASE(
   CHECK(dict == dictionary_ref);
 }
 
+
 TEST_CASE(
     "Compression-Dictionary: Test compression of empty strings",
     "[compression][dict]") {
@@ -299,7 +300,8 @@ TEST_CASE(
           exp_compressed.size()) == 0);
 
   // Decompress the previously compressed array
-  const char* exp_decompressed = "";
+  const char* exp_decompressed =
+      "";
 
   // In this test we allocate atleast 1 byte to avoid empty input buffer
   std::vector<std::byte> decompressed(1);
@@ -321,6 +323,7 @@ TEST_CASE(
   }
 }
 
+
 TEST_CASE(
     "Compression-Dictionary: Test compression of mixed empty strings",
     "[compression][dict]") {
