@@ -89,6 +89,27 @@ enum class VFSMode : uint8_t;
  */
 class VFS {
  public:
+  /* ********************************* */
+  /*          TYPE DEFINITIONS         */
+  /* ********************************* */
+  /**
+   * Multipart upload state definition used in the serialization of remote
+   * global order writes. This state is a generalization of
+   * the multipart upload state types currently defined independently by each
+   * backend implementation.
+   */
+  struct MultiPartUploadState {
+    struct CompletedParts {
+      optional<std::string> e_tag;
+      uint64_t part_number;
+    };
+
+    uint64_t part_number;
+    optional<std::string> upload_id;
+    std::vector<CompletedParts> completed_parts;
+    Status status;
+  };
+
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
@@ -359,6 +380,24 @@ class VFS {
       std::vector<ThreadPool::Task>* tasks,
       bool use_read_ahead = true);
 
+  /**
+   * Reads multiple regions from a file with no batching.
+   *
+   * @param uri The URI of the file.
+   * @param regions The list of regions to read. Each region is a tuple
+   *    `(file_offset, dest_buffer, nbytes)`.
+   * @param thread_pool Thread pool to execute async read tasks to.
+   * @param tasks Vector to which new async read tasks are pushed.
+   * @param use_read_ahead Whether to use the read-ahead cache.
+   * @return Status
+   */
+  Status read_all_no_batching(
+      const URI& uri,
+      const std::vector<tuple<uint64_t, Tile*, uint64_t>>& regions,
+      ThreadPool* thread_pool,
+      std::vector<ThreadPool::Task>* tasks,
+      bool use_read_ahead = true);
+
   /** Checks if a given filesystem is supported. */
   bool supports_fs(Filesystem fs) const;
 
@@ -412,6 +451,37 @@ class VFS {
    */
   Status write(const URI& uri, const void* buffer, uint64_t buffer_size);
 
+  /**
+   * Used in serialization to share the multipart upload state
+   * among cloud executors during global order writes
+   *
+   * @param uri The file uri used as key in the internal map of the backend
+   * @return A pair of status and VFS::MultiPartUploadState object.
+   */
+  std::pair<Status, std::optional<MultiPartUploadState>> multipart_upload_state(
+      const URI& uri);
+
+  /**
+   * Used in serialization of global order writes to set the multipart upload
+   * state in the internal maps of cloud backends during deserialization
+   *
+   * @param uri The file uri used as key in the internal map of the backend
+   * @param state The multipart upload state info
+   * @return Status
+   */
+  Status set_multipart_upload_state(
+      const URI& uri, const MultiPartUploadState& state);
+
+  /**
+   * Used in remote global order writes to flush the internal
+   * in-memory buffer for an URI that backends maintain to modulate the
+   * frequency of multipart upload requests.
+   *
+   * @param uri The file uri identifying the backend file buffer
+   * @return Status
+   */
+  Status flush_multipart_file_buffer(const URI& uri);
+
  private:
   /* ********************************* */
   /*        PRIVATE DATATYPES          */
