@@ -45,9 +45,11 @@
 #include "positive_delta_filter.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/sm/crypto/encryption_key.h"
+#include "tiledb/sm/enums/compressor.h"
 #include "tiledb/sm/enums/encryption_type.h"
 #include "tiledb/sm/enums/filter_type.h"
 #include "tiledb/stdx/utility/to_underlying.h"
+#include "xor_filter.h"
 
 tiledb::sm::Filter* tiledb::sm::FilterCreate::make(FilterType type) {
   switch (type) {
@@ -77,6 +79,8 @@ tiledb::sm::Filter* tiledb::sm::FilterCreate::make(FilterType type) {
       return tdb_new(tiledb::sm::ChecksumSHA256Filter);
     case tiledb::sm::FilterType::FILTER_SCALE_FLOAT:
       return tdb_new(tiledb::sm::FloatScalingFilter);
+    case tiledb::sm::FilterType::FILTER_XOR:
+      return tdb_new(tiledb::sm::XORFilter);
     default:
       throw StatusException(
           "FilterCreate",
@@ -84,34 +88,23 @@ tiledb::sm::Filter* tiledb::sm::FilterCreate::make(FilterType type) {
   }
 }
 
-tuple<Status, optional<shared_ptr<tiledb::sm::Filter>>>
-tiledb::sm::FilterCreate::deserialize(
-    ConstBuffer* buff,
+shared_ptr<tiledb::sm::Filter> tiledb::sm::FilterCreate::deserialize(
+    Deserializer& deserializer,
     const EncryptionKey& encryption_key,
     const uint32_t version) {
   Status st;
-  uint8_t type;
-  st = buff->read(&type, sizeof(uint8_t));
-  if (!st.ok()) {
-    return {st, nullopt};
-  }
+  uint8_t type = deserializer.read<uint8_t>();
   FilterType filtertype = static_cast<FilterType>(type);
-  uint32_t filter_metadata_len;
-  st = buff->read(&filter_metadata_len, sizeof(uint32_t));
-  if (!st.ok()) {
-    return {st, nullopt};
+  uint32_t filter_metadata_len = deserializer.read<uint32_t>();
+  if (deserializer.size() < filter_metadata_len) {
+    throw StatusException(
+        "FilterCreate",
+        "Deserialization error; not enough data in buffer for metadata");
   }
 
-  auto offset = buff->offset();
-  if ((buff->size() - offset) < filter_metadata_len) {
-    return {
-        Status_FilterError(
-            "Deserialization error; not enough data in buffer for metadata"),
-        nullopt};
-  }
   switch (filtertype) {
     case FilterType::FILTER_NONE:
-      return {Status::Ok(), make_shared<NoopFilter>(HERE())};
+      return make_shared<NoopFilter>(HERE());
     case FilterType::FILTER_GZIP:
     case FilterType::FILTER_ZSTD:
     case FilterType::FILTER_LZ4:
@@ -119,80 +112,55 @@ tiledb::sm::FilterCreate::deserialize(
     case FilterType::FILTER_BZIP2:
     case FilterType::FILTER_DOUBLE_DELTA:
     case FilterType::FILTER_DICTIONARY: {
-      uint8_t compressor_char;
-      int compression_level;
-      st = (buff->read(&compressor_char, sizeof(uint8_t)));
-      if (!st.ok()) {
-        return {st, nullopt};
-      }
+      uint8_t compressor_char = deserializer.read<uint8_t>();
+      int compression_level = deserializer.read<int32_t>();
       Compressor compressor = static_cast<Compressor>(compressor_char);
-      st = buff->read(&compression_level, sizeof(int32_t));
-      if (!st.ok()) {
-        return {st, nullopt};
-      }
-      return {Status::Ok(),
-              make_shared<CompressionFilter>(
-                  HERE(), compressor, compression_level, version)};
+      return make_shared<CompressionFilter>(
+          HERE(), compressor, compression_level, version);
     }
     case FilterType::FILTER_BIT_WIDTH_REDUCTION: {
-      uint32_t max_window_size;
-      st = buff->read(&max_window_size, sizeof(uint32_t));
-      if (!st.ok()) {
-        return {st, nullopt};
-      }
-      return {Status::Ok(),
-              make_shared<BitWidthReductionFilter>(HERE(), max_window_size)};
+      uint32_t max_window_size = deserializer.read<uint32_t>();
+      return make_shared<BitWidthReductionFilter>(HERE(), max_window_size);
     }
     case FilterType::FILTER_BITSHUFFLE:
-      return {Status::Ok(), make_shared<BitshuffleFilter>(HERE())};
+      return make_shared<BitshuffleFilter>(HERE());
     case FilterType::FILTER_BYTESHUFFLE:
-      return {Status::Ok(), make_shared<ByteshuffleFilter>(HERE())};
+      return make_shared<ByteshuffleFilter>(HERE());
     case FilterType::FILTER_POSITIVE_DELTA: {
-      uint32_t max_window_size;
-      st = buff->read(&max_window_size, sizeof(uint32_t));
-      if (!st.ok()) {
-        return {st, nullopt};
-      } else {
-        return {Status::Ok(),
-                make_shared<PositiveDeltaFilter>(HERE(), max_window_size)};
-      }
+      uint32_t max_window_size = deserializer.read<uint32_t>();
+      return make_shared<PositiveDeltaFilter>(HERE(), max_window_size);
     }
     case FilterType::INTERNAL_FILTER_AES_256_GCM:
       if (encryption_key.encryption_type() ==
           tiledb::sm::EncryptionType::AES_256_GCM) {
-        return {Status::Ok(),
-                make_shared<EncryptionAES256GCMFilter>(HERE(), encryption_key)};
+        return make_shared<EncryptionAES256GCMFilter>(HERE(), encryption_key);
       } else {
-        return {Status::Ok(), make_shared<EncryptionAES256GCMFilter>(HERE())};
+        return make_shared<EncryptionAES256GCMFilter>(HERE());
       }
     case FilterType::FILTER_CHECKSUM_MD5:
-      return {Status::Ok(), make_shared<ChecksumMD5Filter>(HERE())};
+      return make_shared<ChecksumMD5Filter>(HERE());
     case FilterType::FILTER_CHECKSUM_SHA256:
-      return {Status::Ok(), make_shared<ChecksumSHA256Filter>(HERE())};
+      return make_shared<ChecksumSHA256Filter>(HERE());
     case FilterType::FILTER_SCALE_FLOAT: {
-      FloatScalingFilter::FilterConfig filter_config;
-      st = buff->read(&filter_config, sizeof(FloatScalingFilter::FilterConfig));
-      if (!st.ok()) {
-        return {st, nullopt};
-      } else {
-        return {Status::Ok(),
-                make_shared<FloatScalingFilter>(
-                    HERE(),
-                    filter_config.byte_width,
-                    filter_config.scale,
-                    filter_config.offset)};
-      }
+      auto filter_config =
+          deserializer.read<FloatScalingFilter::FilterConfig>();
+      return make_shared<FloatScalingFilter>(
+          HERE(),
+          filter_config.byte_width,
+          filter_config.scale,
+          filter_config.offset);
     };
+    case FilterType::FILTER_XOR: {
+      return make_shared<XORFilter>(HERE());
+    }
     default:
-      assert(false);
-      return {Status_FilterError("Deserialization error; unknown type"),
-              nullopt};
+      throw StatusException(
+          "FilterCreate", "Deserialization error; unknown type");
   }
 }
-
-tuple<Status, optional<shared_ptr<tiledb::sm::Filter>>>
-tiledb::sm::FilterCreate::deserialize(
-    ConstBuffer* buff, const uint32_t version) {
+shared_ptr<tiledb::sm::Filter> tiledb::sm::FilterCreate::deserialize(
+    Deserializer& deserializer, const uint32_t version) {
   EncryptionKey encryption_key;
-  return tiledb::sm::FilterCreate::deserialize(buff, encryption_key, version);
+  return tiledb::sm::FilterCreate::deserialize(
+      deserializer, encryption_key, version);
 }
\ No newline at end of file
