@@ -114,7 +114,7 @@ class FragmentMetadata {
    */
   inline uint64_t num_dims_and_attrs() const {
     return array_schema_->attribute_num() + array_schema_->dim_num() + 1 +
-           has_timestamps_;
+           has_timestamps_ + (has_delete_meta_ * 2);
   }
 
   /** Returns the number of cells in the fragment. */
@@ -214,26 +214,36 @@ class FragmentMetadata {
   std::vector<std::pair<uint64_t, double>> compute_overlapping_tile_ids_cov(
       const T* subarray) const;
 
+  /** Retrieves the fragment size. */
+  Status fragment_size(uint64_t* size) const;
+
   /**
    * Returns true if the corresponding fragment is dense, and false if it
    * is sparse.
    */
-  bool dense() const;
+  inline bool dense() const {
+    return dense_;
+  }
 
   /** Returns the (expanded) domain in which the fragment is constrained. */
-  const NDRange& domain() const;
+  inline const NDRange& domain() const {
+    return domain_;
+  }
 
   /** Returns the format version of this fragment. */
-  uint32_t format_version() const;
-
-  /** Retrieves the fragment size. */
-  Status fragment_size(uint64_t* size) const;
+  inline uint32_t format_version() const {
+    return version_;
+  }
 
   /** Returns the fragment URI. */
-  const URI& fragment_uri() const;
+  inline const URI& fragment_uri() const {
+    return fragment_uri_;
+  }
 
   /** Returns true if the metadata footer is consolidated. */
-  bool has_consolidated_footer() const;
+  inline bool has_consolidated_footer() const {
+    return has_consolidated_footer_;
+  }
 
   /** Returns true if the fragment has timestamps. */
   inline bool has_timestamps() const {
@@ -245,6 +255,122 @@ class FragmentMetadata {
     return has_delete_meta_;
   }
 
+  /** Returns the sizes of each attribute file. */
+  inline const std::vector<uint64_t>& file_sizes() const {
+    return file_sizes_;
+  }
+
+  /** Returns the sizes of each variable attribute file. */
+  inline const std::vector<uint64_t>& file_var_sizes() const {
+    return file_var_sizes_;
+  }
+
+  /** Returns the sizes of each validity attribute file. */
+  inline const std::vector<uint64_t>& file_validity_sizes() const {
+    return file_validity_sizes_;
+  }
+
+  /** Returns the number of sparse tiles. */
+  inline const uint64_t& sparse_tile_num() const {
+    return sparse_tile_num_;
+  }
+
+  /** Returns the tile index base value. */
+  inline const uint64_t& tile_index_base() const {
+    return tile_index_base_;
+  }
+
+  /** Returns the tile offsets. */
+  inline const std::vector<std::vector<uint64_t>>& tile_offsets() const {
+    return tile_offsets_;
+  }
+
+  /** Returns the variable tile offsets. */
+  inline const std::vector<std::vector<uint64_t>>& tile_var_offsets() const {
+    return tile_var_offsets_;
+  }
+
+  /** Returns the sizes of the uncompressed variable tiles. */
+  inline const std::vector<std::vector<uint64_t>>& tile_var_sizes() const {
+    return tile_var_sizes_;
+  }
+
+  /** Returns the validity tile offsets. */
+  inline const std::vector<std::vector<uint64_t>>& tile_validity_offsets()
+      const {
+    return tile_validity_offsets_;
+  }
+
+  /** Returns the tile min buffers. */
+  inline const std::vector<std::vector<uint8_t>>& tile_min_buffer() const {
+    return tile_min_buffer_;
+  }
+
+  /** Returns the tile min buffers variable length data. */
+  inline const std::vector<std::vector<char>>& tile_min_var_buffer() const {
+    return tile_min_var_buffer_;
+  }
+
+  /** Returns the tile max buffers. */
+  inline const std::vector<std::vector<uint8_t>>& tile_max_buffer() const {
+    return tile_max_buffer_;
+  }
+
+  /** Returns the tile max buffers variable length data. */
+  inline const std::vector<std::vector<char>>& tile_max_var_buffer() const {
+    return tile_max_var_buffer_;
+  }
+
+  /** Returns the tile sum values for fixed sized data. */
+  inline const std::vector<std::vector<uint8_t>>& tile_sums() const {
+    return tile_sums_;
+  }
+
+  /** Returns the tile null count values for attributes/dimensions. */
+  inline const std::vector<std::vector<uint64_t>>& tile_null_counts() const {
+    return tile_null_counts_;
+  }
+
+  /** Returns the fragment mins. */
+  inline const std::vector<std::vector<uint8_t>>& fragment_mins() const {
+    return fragment_mins_;
+  }
+
+  /** Returns the fragment maxs. */
+  inline const std::vector<std::vector<uint8_t>>& fragment_maxs() const {
+    return fragment_maxs_;
+  }
+
+  /** Returns the fragment sums. */
+  inline const std::vector<uint64_t>& fragment_sums() const {
+    return fragment_sums_;
+  }
+
+  /** Returns the fragment null counts. */
+  inline const std::vector<uint64_t>& fragment_null_counts() const {
+    return fragment_null_counts_;
+  }
+
+  /** Returns the fragment timestamp range. */
+  inline const std::pair<uint64_t, uint64_t>& timestamp_range() const {
+    return timestamp_range_;
+  }
+
+  /** Returns the number of cells in the last tile. */
+  inline const uint64_t& last_tile_cell_num() const {
+    return last_tile_cell_num_;
+  }
+
+  /** Returns the non-empty domain of the fragment. */
+  inline const NDRange& non_empty_domain() const {
+    return non_empty_domain_;
+  }
+
+  /** Returns an RTree for the MBRs. */
+  inline const RTree& rtree() const {
+    return rtree_;
+  }
+
   /**
    * Retrieves the overlap of all MBRs with the input ND range.
    */
@@ -268,8 +394,10 @@ class FragmentMetadata {
    */
   Status init(const NDRange& non_empty_domain);
 
-  /** Returns the number of cells in the last tile. */
-  uint64_t last_tile_cell_num() const;
+  /**
+   * Initializes the fragment's internal domain and non-empty domain members
+   */
+  Status init_domain(const NDRange& non_empty_domain);
 
   /**
    * Loads the basic metadata from storage or `f_buff` for later
@@ -282,7 +410,7 @@ class FragmentMetadata {
       std::unordered_map<std::string, shared_ptr<ArraySchema>> array_schemas);
 
   /** Stores all the metadata to storage. */
-  Status store(const EncryptionKey& encryption_key);
+  void store(const EncryptionKey& encryption_key);
 
   /**
    * Stores all the metadata to storage.
@@ -312,9 +440,6 @@ class FragmentMetadata {
    */
   Status store_v15_or_higher(const EncryptionKey& encryption_key);
 
-  /** Returns the non-empty domain in which the fragment is constrained. */
-  const NDRange& non_empty_domain();
-
   /**
    * Simply sets the number of cells for the last tile.
    *
@@ -507,8 +632,11 @@ class FragmentMetadata {
    */
   void set_array_schema(const shared_ptr<const ArraySchema>& array_schema);
 
-  /** Returns the tile index base value. */
-  uint64_t tile_index_base() const;
+  /** Sets the array_schema name */
+  void set_schema_name(const std::string& name);
+
+  /** Sets the internal dense_ field*/
+  void set_dense(bool dense);
 
   /** Returns the number of tiles in the fragment. */
   uint64_t tile_num() const;
@@ -712,6 +840,15 @@ class FragmentMetadata {
    */
   tuple<Status, optional<uint64_t>> get_null_count(const std::string& name);
 
+  /**
+   * Set the processed conditions. The processed conditions is the list
+   * of delete/update conditions that have already been applied for this
+   * fragment and don't need to be applied again.
+   *
+   * @param processed_conditions The processed conditions.
+   */
+  void set_processed_conditions(std::vector<std::string>& processed_conditions);
+
   /**
    * Retrieves the processed conditions. The processed conditions is the list
    * of delete/update conditions that have already been applied for this
@@ -719,14 +856,20 @@ class FragmentMetadata {
    *
    * @return Processed conditions.
    */
-  std::unordered_set<std::string>& get_processed_conditions();
+  std::vector<std::string>& get_processed_conditions();
+
+  /**
+   * Retrieves the processed conditions set. The processed conditions is the
+   * list of delete/update conditions that have already been applied for this
+   * fragment and don't need to be applied again.
+   *
+   * @return Processed conditions set.
+   */
+  std::unordered_set<std::string>& get_processed_conditions_set();
 
   /** Returns the first timestamp of the fragment timestamp range. */
   uint64_t first_timestamp() const;
 
-  /** Returns the fragment timestamp range. */
-  const std::pair<uint64_t, uint64_t>& timestamp_range() const;
-
   /**
    * Returns `true` if the timestamp of the first operand is smaller,
    * breaking ties based on the URI string.
@@ -848,6 +991,156 @@ class FragmentMetadata {
    */
   const shared_ptr<const ArraySchema>& array_schema() const;
 
+  /** File sizes accessor */
+  std::vector<uint64_t>& file_sizes() {
+    return file_sizes_;
+  }
+
+  /** File var sizes accessor */
+  std::vector<uint64_t>& file_var_sizes() {
+    return file_var_sizes_;
+  }
+
+  /** File validity sizes accessor */
+  std::vector<uint64_t>& file_validity_sizes() {
+    return file_validity_sizes_;
+  }
+
+  /** Fragment uri accessor */
+  URI& fragment_uri() {
+    return fragment_uri_;
+  }
+
+  /** has_timestamps accessor */
+  bool& has_timestamps() {
+    return has_timestamps_;
+  }
+
+  /** has_delete_meta accessor */
+  bool& has_delete_meta() {
+    return has_delete_meta_;
+  }
+
+  /** has_consolidated_footer accessor */
+  bool& has_consolidated_footer() {
+    return has_consolidated_footer_;
+  }
+
+  /** sparse_tile_num accessor */
+  uint64_t& sparse_tile_num() {
+    return sparse_tile_num_;
+  }
+
+  /** tile_index_base accessor */
+  uint64_t& tile_index_base() {
+    return tile_index_base_;
+  }
+
+  /** tile_offsets accessor */
+  std::vector<std::vector<uint64_t>>& tile_offsets() {
+    return tile_offsets_;
+  }
+
+  /** tile_var_offsets accessor */
+  std::vector<std::vector<uint64_t>>& tile_var_offsets() {
+    return tile_var_offsets_;
+  }
+
+  /** tile_var_sizes  accessor */
+  std::vector<std::vector<uint64_t>>& tile_var_sizes() {
+    return tile_var_sizes_;
+  }
+
+  /** tile_validity_offsets accessor */
+  std::vector<std::vector<uint64_t>>& tile_validity_offsets() {
+    return tile_validity_offsets_;
+  }
+
+  /** tile_min_buffer accessor */
+  std::vector<std::vector<uint8_t>>& tile_min_buffer() {
+    return tile_min_buffer_;
+  }
+
+  /** tile_min_var_buffer accessor */
+  std::vector<std::vector<char>>& tile_min_var_buffer() {
+    return tile_min_var_buffer_;
+  }
+
+  /** tile_max_buffer accessor */
+  std::vector<std::vector<uint8_t>>& tile_max_buffer() {
+    return tile_max_buffer_;
+  }
+
+  /** tile_max_var_buffer accessor */
+  std::vector<std::vector<char>>& tile_max_var_buffer() {
+    return tile_max_var_buffer_;
+  }
+
+  /** tile_sums accessor */
+  std::vector<std::vector<uint8_t>>& tile_sums() {
+    return tile_sums_;
+  }
+
+  /** tile_null_counts accessor */
+  std::vector<std::vector<uint64_t>>& tile_null_counts() {
+    return tile_null_counts_;
+  }
+
+  /** fragment_mins accessor */
+  std::vector<std::vector<uint8_t>>& fragment_mins() {
+    return fragment_mins_;
+  }
+
+  /** fragment_maxs accessor */
+  std::vector<std::vector<uint8_t>>& fragment_maxs() {
+    return fragment_maxs_;
+  }
+
+  /** fragment_sums accessor */
+  std::vector<uint64_t>& fragment_sums() {
+    return fragment_sums_;
+  }
+
+  /** fragment_null_counts accessor */
+  std::vector<uint64_t>& fragment_null_counts() {
+    return fragment_null_counts_;
+  }
+
+  /** version accessor */
+  uint32_t& version() {
+    return version_;
+  }
+
+  /** timestamp_range accessor */
+  std::pair<uint64_t, uint64_t>& timestamp_range() {
+    return timestamp_range_;
+  }
+
+  /** last_tile_cell_num accessor */
+  uint64_t& last_tile_cell_num() {
+    return last_tile_cell_num_;
+  }
+
+  /** non_empty_domain accessor */
+  NDRange& non_empty_domain() {
+    return non_empty_domain_;
+  }
+
+  /** rtree accessor */
+  RTree& rtree() {
+    return rtree_;
+  }
+
+  /** set the SM pointer during deserialization*/
+  void set_storage_manager(StorageManager* sm) {
+    storage_manager_ = sm;
+  }
+
+  /** loaded_metadata_.rtree_ accessor */
+  void set_rtree_loaded() {
+    loaded_metadata_.rtree_ = true;
+  }
+
  private:
   /* ********************************* */
   /*          TYPE DEFINITIONS         */
@@ -1076,7 +1369,13 @@ class FragmentMetadata {
   URI array_uri_;
 
   /** Set of already processed delete/update conditions for this fragment. */
-  std::unordered_set<std::string> processed_conditions_;
+  std::unordered_set<std::string> processed_conditions_set_;
+
+  /**
+   * Ordered list of already processed delete/update conditions for this
+   * fragment.
+   */
+  std::vector<std::string> processed_conditions_;
 
   /* ********************************* */
   /*           PRIVATE METHODS         */
@@ -1222,9 +1521,15 @@ class FragmentMetadata {
 
   /**
    * Loads the generic tile offsets from the buffer. Applicable to
-   * versions 12 or higher.
+   * versions 12 to 15.
    */
-  Status load_generic_tile_offsets_v12_or_higher(ConstBuffer* buff);
+  Status load_generic_tile_offsets_v12_v15(ConstBuffer* buff);
+
+  /**
+   * Loads the generic tile offsets from the buffer. Applicable to
+   * versions 16 or higher.
+   */
+  Status load_generic_tile_offsets_v16_or_higher(ConstBuffer* buff);
 
   /**
    * Loads the array schema name.
@@ -1332,13 +1637,13 @@ class FragmentMetadata {
    * Loads the tile offsets for the input attribute from the input buffer.
    * Applicable to versions 1 and 2
    */
-  Status load_tile_offsets(ConstBuffer* buff);
+  Status load_tile_offsets(ConstBuffer* cbuff);
 
   /**
    * Loads the tile offsets for the input attribute or dimension from the
    * input buffer.
    */
-  Status load_tile_offsets(unsigned idx, ConstBuffer* buff);
+  void load_tile_offsets(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the variable tile offsets from the input buffer.
@@ -1350,7 +1655,7 @@ class FragmentMetadata {
    * Loads the variable tile offsets for the input attribute or dimension from
    * the input buffer.
    */
-  Status load_tile_var_offsets(unsigned idx, ConstBuffer* buff);
+  void load_tile_var_offsets(unsigned idx, Deserializer& deserializer);
 
   /** Loads the variable tile sizes from the input buffer. */
   Status load_tile_var_sizes(ConstBuffer* buff);
@@ -1359,7 +1664,7 @@ class FragmentMetadata {
    * Loads the variable tile sizes for the input attribute or dimension
    * from the input buffer.
    */
-  Status load_tile_var_sizes(unsigned idx, ConstBuffer* buff);
+  void load_tile_var_sizes(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the validity tile offsets for the input attribute from the
@@ -1370,33 +1675,34 @@ class FragmentMetadata {
   /**
    * Loads the min values for the input attribute from the input buffer.
    */
-  Status load_tile_min_values(unsigned idx, ConstBuffer* buff);
+  void load_tile_min_values(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the max values for the input attribute from the input buffer.
    */
-  Status load_tile_max_values(unsigned idx, ConstBuffer* buff);
+  // Status load_tile_max_values(unsigned idx, ConstBuffer* buff);
+  void load_tile_max_values(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the sum values for the input attribute from the input buffer.
    */
-  Status load_tile_sum_values(unsigned idx, ConstBuffer* buff);
+  void load_tile_sum_values(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the null count values for the input attribute from the input
    * buffer.
    */
-  Status load_tile_null_count_values(unsigned idx, ConstBuffer* buff);
+  void load_tile_null_count_values(unsigned idx, Deserializer& deserializer);
 
   /**
    * Loads the min max sum null count values for the fragment.
    */
-  Status load_fragment_min_max_sum_null_count(ConstBuffer* buff);
+  void load_fragment_min_max_sum_null_count(Deserializer& deserializer);
 
   /**
    * Loads the processed conditions for the fragment.
    */
-  Status load_processed_conditions(ConstBuffer* buff);
+  void load_processed_conditions(Deserializer& deserializer);
 
   /** Loads the format version from the buffer. */
   Status load_version(ConstBuffer* buff);
@@ -1477,8 +1783,8 @@ class FragmentMetadata {
   /** Stores a footer with the basic information. */
   Status store_footer(const EncryptionKey& encryption_key);
 
-  /** Writes the R-tree to the input buffer. */
-  Status write_rtree(Buffer* buff);
+  /** Writes the R-tree to a tile. */
+  Tile write_rtree();
 
   /** Writes the non-empty domain to the input buffer. */
   Status write_non_empty_domain(Buffer* buff) const;
@@ -1491,14 +1797,14 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the tile offsets.
    * @return Status
    */
-  Status store_tile_offsets(
+  void store_tile_offsets(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the tile offsets of the input attribute or dimension idx to the
    * input buffer.
    */
-  Status write_tile_offsets(unsigned idx, Buffer* buff);
+  void write_tile_offsets(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the variable tile offsets of the input attribute or dimension
@@ -1509,14 +1815,14 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the tile var offsets.
    * @return Status
    */
-  Status store_tile_var_offsets(
+  void store_tile_var_offsets(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the variable tile offsets of the input attribute or dimension idx
    * to the buffer.
    */
-  Status write_tile_var_offsets(unsigned idx, Buffer* buff);
+  void write_tile_var_offsets(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the variable tile sizes for the input attribute or dimension to
@@ -1527,14 +1833,14 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the tile var sizes.
    * @return Status
    */
-  Status store_tile_var_sizes(
+  void store_tile_var_sizes(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the variable tile sizes for the input attribute or dimension
    * to storage.
    */
-  Status write_tile_var_sizes(unsigned idx, Buffer* buff);
+  void write_tile_var_sizes(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the validity tile offsets of the input attribute to storage.
@@ -1562,13 +1868,13 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the mins.
    * @return Status
    */
-  Status store_tile_mins(
+  void store_tile_mins(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the mins of the input attribute idx to the input buffer.
    */
-  Status write_tile_mins(unsigned idx, Buffer* buff);
+  void write_tile_mins(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the maxs of the input attribute to storage.
@@ -1578,13 +1884,13 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the maxs.
    * @return Status
    */
-  Status store_tile_maxs(
+  void store_tile_maxs(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the maxs of the input attribute idx to the input buffer.
    */
-  Status write_tile_maxs(unsigned idx, Buffer* buff);
+  void write_tile_maxs(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the sums of the input attribute to storage.
@@ -1594,13 +1900,13 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the sums.
    * @return Status
    */
-  Status store_tile_sums(
+  void store_tile_sums(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the sums of the input attribute idx to the input buffer.
    */
-  Status write_tile_sums(unsigned idx, Buffer* buff);
+  void write_tile_sums(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the null counts of the input attribute to storage.
@@ -1610,13 +1916,13 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written for the null counts.
    * @return Status
    */
-  Status store_tile_null_counts(
+  void store_tile_null_counts(
       unsigned idx, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
    * Writes the null counts of the input attribute idx to the input buffer.
    */
-  Status write_tile_null_counts(unsigned idx, Buffer* buff);
+  void write_tile_null_counts(unsigned idx, Serializer& serializer);
 
   /**
    * Writes the fragment min, max, sum and null count to storage.
@@ -1624,9 +1930,8 @@ class FragmentMetadata {
    * @param num The number of attributes.
    * @param encryption_key The encryption key.
    * @param nbytes The total number of bytes written.
-   * @return Status
    */
-  Status store_fragment_min_max_sum_null_count(
+  void store_fragment_min_max_sum_null_count(
       uint64_t num, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
@@ -1636,7 +1941,7 @@ class FragmentMetadata {
    * @param nbytes The total number of bytes written.
    * @return Status
    */
-  Status store_processed_conditions(
+  void store_processed_conditions(
       const EncryptionKey& encryption_key, uint64_t* nbytes);
 
   /**
@@ -1674,9 +1979,9 @@ class FragmentMetadata {
 
   /**
    * Reads the contents of a generic tile starting at the input offset,
-   * and stores them into buffer ``buff``.
+   * and returns a tile.
    */
-  tuple<Status, optional<Buffer>> read_generic_tile_from_file(
+  tuple<Status, optional<Tile>> read_generic_tile_from_file(
       const EncryptionKey& encryption_key, uint64_t offset) const;
 
   /**
@@ -1700,6 +2005,18 @@ class FragmentMetadata {
       Buffer& buff,
       uint64_t* nbytes) const;
 
+  /**
+   * Writes the contents of the input tile as a separate
+   * generic tile to the metadata file.
+   *
+   * @param encryption_key The encryption key.
+   * @param tile The tile whose contents the function will write.
+   * @param nbytes The total number of bytes written to the file.
+   * @return Status
+   */
+  Status write_generic_tile_to_file(
+      const EncryptionKey& encryption_key, Tile& tile, uint64_t* nbytes) const;
+
   /**
    * Writes the contents of the input buffer at the end of the fragment
    * metadata file, without applying any filters. This helps its quick
