@@ -63,6 +63,7 @@
 #include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/misc/uuid.h"
+#include "tiledb/sm/query/deletes_and_updates/serialization.h"
 #include "tiledb/sm/query/query.h"
 #include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/stats/global_stats.h"
@@ -149,6 +150,26 @@ Status StorageManager::array_close_for_writes(Array* array) {
   return Status::Ok();
 }
 
+Status StorageManager::array_close_for_deletes(Array* array) {
+  assert(open_arrays_.find(array) != open_arrays_.end());
+
+  // Remove entry from open arrays
+  std::lock_guard<std::mutex> lock{open_arrays_mtx_};
+  open_arrays_.erase(array);
+
+  return Status::Ok();
+}
+
+Status StorageManager::array_close_for_updates(Array* array) {
+  assert(open_arrays_.find(array) != open_arrays_.end());
+
+  // Remove entry from open arrays
+  std::lock_guard<std::mutex> lock{open_arrays_mtx_};
+  open_arrays_.erase(array);
+
+  return Status::Ok();
+}
+
 Status StorageManager::group_close_for_reads(Group* group) {
   assert(open_groups_.find(group) != open_groups_.end());
 
@@ -332,16 +353,17 @@ StorageManager::array_open_for_writes(Array* array) {
               nullopt,
               nullopt};
     }
-  }
-  if (version > constants::format_version) {
-    std::stringstream err;
-    err << "Cannot open array for writes; Array format version (";
-    err << version;
-    err << ") is newer than library format version (";
-    err << constants::format_version << ")";
-    return {logger_->status(Status_StorageManagerError(err.str())),
-            nullopt,
-            nullopt};
+  } else {
+    if (version > constants::format_version) {
+      std::stringstream err;
+      err << "Cannot open array for writes; Array format version (";
+      err << version;
+      err << ") is newer than library format version (";
+      err << constants::format_version << ")";
+      return {logger_->status(Status_StorageManagerError(err.str())),
+              nullopt,
+              nullopt};
+    }
   }
 
   // Mark the array as open
@@ -540,6 +562,79 @@ Status StorageManager::fragments_consolidate(
       array_name, encryption_type, encryption_key, key_length, fragment_uris);
 }
 
+Status StorageManager::write_commit_ignore_file(
+    ArrayDirectory array_dir, const std::vector<URI>& commit_uris_to_ignore) {
+  auto&& [st, name] = array_dir.compute_new_fragment_name(
+      commit_uris_to_ignore.front(),
+      commit_uris_to_ignore.back(),
+      constants::format_version);
+  RETURN_NOT_OK(st);
+
+  // Write URIs, relative to the array URI.
+  std::stringstream ss;
+  auto base_uri_size = array_dir.uri().to_string().size();
+  for (const auto& uri : commit_uris_to_ignore) {
+    ss << uri.to_string().substr(base_uri_size) << "\n";
+  }
+
+  // Write an ignore file to ensure consolidated WRT files still work
+  auto data = ss.str();
+  URI ignore_file_uri =
+      array_dir.get_commits_dir(constants::format_version)
+          .join_path(name.value() + constants::ignore_file_suffix);
+  RETURN_NOT_OK(vfs_->write(ignore_file_uri, data.c_str(), data.size()));
+  RETURN_NOT_OK(vfs_->close_file(ignore_file_uri));
+
+  return Status::Ok();
+}
+
+Status StorageManager::delete_fragments(
+    const char* array_name, uint64_t timestamp_start, uint64_t timestamp_end) {
+  Status st;
+  if (array_name == nullptr) {
+    throw Status_StorageManagerError(
+        "Cannot delete_fragments; Array name cannot be null");
+  }
+
+  auto array_dir = ArrayDirectory(
+      vfs_, compute_tp_, URI(array_name), timestamp_start, timestamp_end);
+
+  // Get the fragment URIs to be deleted
+  auto filtered_fragment_uris = array_dir.filtered_fragment_uris(true);
+  const auto& fragment_uris = filtered_fragment_uris.fragment_uris();
+
+  // Retrieve commit uris to delete and ignore
+  std::vector<URI> commit_uris_to_delete;
+  std::vector<URI> commit_uris_to_ignore;
+  for (auto& fragment : fragment_uris) {
+    auto commit_uri = array_dir.get_commit_uri(fragment.uri_);
+    commit_uris_to_delete.emplace_back(commit_uri);
+    if (array_dir.consolidated_commit_uris_set().count(commit_uri.c_str()) !=
+        0) {
+      commit_uris_to_ignore.emplace_back(commit_uri);
+    }
+  }
+
+  // Write ignore file
+  if (commit_uris_to_ignore.size() != 0) {
+    RETURN_NOT_OK(write_commit_ignore_file(array_dir, commit_uris_to_ignore));
+  }
+
+  // Delete fragments and commits
+  st = parallel_for(compute_tp(), 0, fragment_uris.size(), [&](size_t i) {
+    RETURN_NOT_OK(vfs_->remove_dir(fragment_uris[i].uri_));
+    bool is_file = false;
+    RETURN_NOT_OK(vfs_->is_file(commit_uris_to_delete[i], &is_file));
+    if (is_file) {
+      RETURN_NOT_OK(vfs_->remove_file(commit_uris_to_delete[i]));
+    }
+    return Status::Ok();
+  });
+  RETURN_NOT_OK(st);
+
+  return Status::Ok();
+}
+
 Status StorageManager::array_vacuum(
     const char* array_name, const Config* config) {
   // If 'config' is unset, use the 'config_' that was set during initialization
@@ -633,8 +728,7 @@ Status StorageManager::array_create(
   }
 
   // Check if array exists
-  bool exists = false;
-  RETURN_NOT_OK(is_array(array_uri, &exists));
+  bool exists = is_array(array_uri);
   if (exists)
     return logger_->status(Status_StorageManagerError(
         std::string("Cannot create array; Array '") + array_uri.c_str() +
@@ -672,6 +766,12 @@ Status StorageManager::array_create(
       array_uri.join_path(constants::array_fragment_meta_dir_name);
   RETURN_NOT_OK(vfs_->create_dir(array_fragment_metadata_uri));
 
+  if constexpr (is_experimental_build) {
+    URI array_dimension_labels_uri =
+        array_uri.join_path(constants::array_dimension_labels_dir_name);
+    RETURN_NOT_OK(vfs_->create_dir(array_dimension_labels_uri));
+  }
+
   // Get encryption key from config
   Status st;
   if (encryption_key.encryption_type() == EncryptionType::NO_ENCRYPTION) {
@@ -722,11 +822,9 @@ Status StorageManager::array_create(
 }
 
 Status StorageManager::array_evolve_schema(
-    const ArrayDirectory& array_dir,
+    const URI& array_uri,
     ArraySchemaEvolution* schema_evolution,
     const EncryptionKey& encryption_key) {
-  const URI& array_uri = array_dir.uri();
-
   // Check array schema
   if (schema_evolution == nullptr) {
     return logger_->status(Status_StorageManagerError(
@@ -738,13 +836,21 @@ Status StorageManager::array_evolve_schema(
         array_uri, schema_evolution);
   }
 
+  // Load URIs from the array directory
+  tiledb::sm::ArrayDirectory array_dir{
+      this->vfs(),
+      this->io_tp(),
+      array_uri,
+      0,
+      UINT64_MAX,
+      tiledb::sm::ArrayDirectoryMode::SCHEMA_ONLY};
+
   // Check if array exists
-  bool exists = false;
-  RETURN_NOT_OK(is_array(array_uri, &exists));
-  if (!exists)
+  if (!is_array(array_uri)) {
     return logger_->status(Status_StorageManagerError(
         std::string("Cannot evolve array; Array '") + array_uri.c_str() +
         "' not exists"));
+  }
 
   auto&& [st1, array_schema] =
       load_array_schema_latest(array_dir, encryption_key);
@@ -766,17 +872,22 @@ Status StorageManager::array_evolve_schema(
 }
 
 Status StorageManager::array_upgrade_version(
-    const ArrayDirectory& array_dir, const Config* config) {
-  const URI& array_uri = array_dir.uri();
-
+    const URI& array_uri, const Config* config) {
   // Check if array exists
-  bool exists = false;
-  RETURN_NOT_OK(is_array(array_uri, &exists));
-  if (!exists)
+  if (!is_array(array_uri))
     return logger_->status(Status_StorageManagerError(
         std::string("Cannot upgrade array; Array '") + array_uri.c_str() +
         "' does not exist"));
 
+  // Load URIs from the array directory
+  tiledb::sm::ArrayDirectory array_dir{
+      this->vfs(),
+      this->io_tp(),
+      array_uri,
+      0,
+      UINT64_MAX,
+      tiledb::sm::ArrayDirectoryMode::SCHEMA_ONLY};
+
   // If 'config' is unset, use the 'config_' that was set during initialization
   // of this StorageManager instance.
   if (!config) {
@@ -786,10 +897,10 @@ Status StorageManager::array_upgrade_version(
   // Get encryption key from config
   bool found = false;
   std::string encryption_key_from_cfg =
-      config_.get("sm.encryption_key", &found);
+      config->get("sm.encryption_key", &found);
   assert(found);
   std::string encryption_type_from_cfg =
-      config_.get("sm.encryption_type", &found);
+      config->get("sm.encryption_type", &found);
   assert(found);
   auto [st1, etc] = encryption_type_enum(encryption_type_from_cfg);
   RETURN_NOT_OK(st1);
@@ -870,8 +981,7 @@ Status StorageManager::array_get_non_empty_domain(
     return logger_->status(Status_StorageManagerError(
         "Cannot get non-empty domain; Array is not open"));
 
-  QueryType query_type;
-  RETURN_NOT_OK(array->get_query_type(&query_type));
+  QueryType query_type{array->get_query_type()};
   if (query_type != QueryType::READ)
     return logger_->status(Status_StorageManagerError(
         "Cannot get non-empty domain; Array not opened for reads"));
@@ -1368,29 +1478,31 @@ void StorageManager::increment_in_progress() {
   queries_in_progress_cv_.notify_all();
 }
 
-Status StorageManager::is_array(const URI& uri, bool* is_array) const {
+bool StorageManager::is_array(const URI& uri) const {
   // Handle remote array
   if (uri.is_tiledb()) {
     auto&& [st, exists] = rest_client_->check_array_exists_from_rest(uri);
-    RETURN_NOT_OK(st);
-    *is_array = *exists;
+    throw_if_not_ok(st);
+    assert(exists.has_value());
+    return exists.value();
   } else {
     // Check if the schema directory exists or not
-    bool is_dir = false;
-    // Since is_dir could return NOT Ok status, we will not use RETURN_NOT_OK
-    // here
-    Status st =
-        vfs_->is_dir(uri.join_path(constants::array_schema_dir_name), &is_dir);
-    if (st.ok() && is_dir) {
-      *is_array = true;
-      return Status::Ok();
+    bool dir_exists = false;
+    throw_if_not_ok(vfs_->is_dir(
+        uri.join_path(constants::array_schema_dir_name), &dir_exists));
+
+    if (dir_exists) {
+      return true;
     }
 
+    bool schema_exists = false;
     // If there is no schema directory, we check schema file
-    RETURN_NOT_OK(vfs_->is_file(
-        uri.join_path(constants::array_schema_filename), is_array));
+    throw_if_not_ok(vfs_->is_file(
+        uri.join_path(constants::array_schema_filename), &schema_exists));
+    return schema_exists;
   }
-  return Status::Ok();
+
+  // TODO: mark unreachable
 }
 
 Status StorageManager::is_file(const URI& uri, bool* is_file) const {
@@ -1425,21 +1537,20 @@ StorageManager::load_array_schema_from_uri(
     const URI& schema_uri, const EncryptionKey& encryption_key) {
   auto timer_se = stats_->start_timer("sm_load_array_schema_from_uri");
 
-  auto&& [st, buffer_opt] =
-      load_data_from_generic_tile(schema_uri, encryption_key);
+  auto&& [st, tile_opt] =
+      load_data_from_generic_tile(schema_uri, 0, encryption_key);
   RETURN_NOT_OK_TUPLE(st, nullopt);
-  auto& buffer = *buffer_opt;
+  auto& tile = *tile_opt;
 
-  stats_->add_counter("read_array_schema_size", buffer.size());
+  stats_->add_counter("read_array_schema_size", tile.size());
 
   // Deserialize
-  ConstBuffer cbuff(&buffer);
-  auto deserialized_schema{ArraySchema::deserialize(&cbuff, schema_uri)};
+  Deserializer deserializer(tile.data(), tile.size());
 
-  // #TODO Update catch statement after later StatusException changes
   try {
     return {Status::Ok(),
-            make_shared<ArraySchema>(HERE(), deserialized_schema)};
+            make_shared<ArraySchema>(
+                HERE(), ArraySchema::deserialize(deserializer, schema_uri))};
   } catch (const StatusException& e) {
     return {Status_StorageManagerError(e.what()), nullopt};
   }
@@ -1514,10 +1625,16 @@ StorageManager::load_all_array_schemas(
   auto status =
       parallel_for(compute_tp_, 0, schema_num, [&](size_t schema_ith) {
         auto& schema_uri = schema_uris[schema_ith];
-        auto&& [st, array_schema] =
-            load_array_schema_from_uri(schema_uri, encryption_key);
-        RETURN_NOT_OK(st);
-        schema_vector[schema_ith] = array_schema.value();
+        try {
+          auto&& [st, array_schema] =
+              load_array_schema_from_uri(schema_uri, encryption_key);
+          RETURN_NOT_OK(st);
+          array_schema.value()->set_array_uri(array_uri);
+          schema_vector[schema_ith] = array_schema.value();
+        } catch (std::exception& e) {
+          return Status_StorageManagerError(e.what());
+        }
+
         return Status::Ok();
       });
   RETURN_NOT_OK_TUPLE(status, nullopt);
@@ -1549,9 +1666,14 @@ Status StorageManager::load_array_metadata(
   auto status = parallel_for(compute_tp_, 0, metadata_num, [&](size_t m) {
     const auto& uri = array_metadata_to_load[m].uri_;
 
-    auto&& [st, buffer] = load_data_from_generic_tile(uri, encryption_key);
+    auto&& [st, tile] = load_data_from_generic_tile(uri, 0, encryption_key);
     RETURN_NOT_OK(st);
-    metadata_buffs[m] = make_shared<Buffer>(HERE(), std::move(*buffer));
+
+    metadata_buffs[m] = tdb::make_shared<Buffer>(HERE());
+    RETURN_NOT_OK(metadata_buffs[m]->realloc(tile->size()));
+    metadata_buffs[m]->set_size(tile->size());
+    RETURN_NOT_OK(
+        tile->read(metadata_buffs[m]->data(), 0, metadata_buffs[m]->size()));
 
     return Status::Ok();
   });
@@ -1563,12 +1685,7 @@ Status StorageManager::load_array_metadata(
     meta_size += b->size();
   stats_->add_counter("read_array_meta_size", meta_size);
 
-  auto&& [st_metadata, deserialized_metadata]{
-      Metadata::deserialize(metadata_buffs)};
-  if (!st_metadata.ok()) {
-    return st_metadata;
-  }
-  *metadata = *(deserialized_metadata.value());
+  *metadata = Metadata::deserialize(metadata_buffs);
 
   // Sets the loaded metadata URIs
   RETURN_NOT_OK(metadata->set_loaded_metadata_uris(array_metadata_to_load));
@@ -1576,6 +1693,35 @@ Status StorageManager::load_array_metadata(
   return Status::Ok();
 }
 
+tuple<Status, optional<std::vector<QueryCondition>>>
+StorageManager::load_delete_conditions(const Array& array) {
+  auto& locations = array.array_directory().delete_tiles_location();
+  auto delete_conditions = std::vector<QueryCondition>(locations.size());
+
+  auto status = parallel_for(compute_tp_, 0, locations.size(), [&](size_t i) {
+    // Get condition marker.
+    auto& uri = locations[i].uri();
+
+    // Read the condition from storage.
+    auto&& [st, tile_opt] = load_data_from_generic_tile(
+        uri, locations[i].offset(), *array.encryption_key());
+    RETURN_NOT_OK(st);
+
+    delete_conditions[i] =
+        tiledb::sm::deletes_and_updates::serialization::deserialize_condition(
+            i,
+            locations[i].condition_marker(),
+            tile_opt->data(),
+            tile_opt->size());
+
+    delete_conditions[i].check(array.array_schema_latest());
+    return Status::Ok();
+  });
+  RETURN_NOT_OK_TUPLE(status, nullopt);
+
+  return {Status::Ok(), delete_conditions};
+}
+
 Status StorageManager::object_type(const URI& uri, ObjectType* type) const {
   URI dir_uri = uri;
   if (uri.is_s3() || uri.is_azure() || uri.is_gcs()) {
@@ -1594,8 +1740,7 @@ Status StorageManager::object_type(const URI& uri, ObjectType* type) const {
       return Status::Ok();
     }
   }
-  bool exists = false;
-  RETURN_NOT_OK(is_array(uri, &exists));
+  bool exists = is_array(uri);
   if (exists) {
     *type = ObjectType::ARRAY;
     return Status::Ok();
@@ -1827,10 +1972,15 @@ Status StorageManager::store_group_detail(
   RETURN_NOT_OK(st);
 
   // Serialize
-  Buffer buff;
-  RETURN_NOT_OK(group->serialize(&buff));
+  SizeComputationSerializer size_computation_serializer;
+  group->serialize(size_computation_serializer);
 
-  stats_->add_counter("write_group_size", buff.size());
+  Tile tile{Tile::from_generic(size_computation_serializer.size())};
+
+  Serializer serializer(tile.data(), tile.size());
+  group->serialize(serializer);
+
+  stats_->add_counter("write_group_size", tile.size());
 
   // Check if the array schema directory exists
   // If not create it, this is caused by a pre-v10 array
@@ -1840,8 +1990,7 @@ Status StorageManager::store_group_detail(
     RETURN_NOT_OK(create_dir(group_detail_folder_uri));
 
   RETURN_NOT_OK(store_data_to_generic_tile(
-      buff.data(), buff.size(), *group_detail_uri, encryption_key));
-  buff.disown_data();
+      tile.data(), tile.size(), *group_detail_uri, encryption_key));
 
   return st;
 }
@@ -1852,10 +2001,14 @@ Status StorageManager::store_array_schema(
   const URI schema_uri = array_schema->uri();
 
   // Serialize
-  Buffer buff;
-  RETURN_NOT_OK(array_schema->serialize(&buff));
+  SizeComputationSerializer size_computation_serializer;
+  array_schema->serialize(size_computation_serializer);
 
-  stats_->add_counter("write_array_schema_size", buff.size());
+  Tile tile{Tile::from_generic(size_computation_serializer.size())};
+  Serializer serializer(tile.data(), tile.size());
+  array_schema->serialize(serializer);
+
+  stats_->add_counter("write_array_schema_size", tile.size());
 
   // Delete file if it exists already
   bool exists;
@@ -1872,9 +2025,7 @@ Status StorageManager::store_array_schema(
   if (!schema_dir_exists)
     RETURN_NOT_OK(create_dir(array_schema_dir_uri));
 
-  RETURN_NOT_OK(store_data_to_generic_tile(
-      buff.data(), buff.size(), schema_uri, encryption_key));
-  buff.disown_data();
+  RETURN_NOT_OK(store_data_to_generic_tile(tile, schema_uri, encryption_key));
 
   return Status::Ok();
 }
@@ -1906,7 +2057,6 @@ Status StorageManager::store_metadata(
       metadata_buff.size(),
       metadata_uri,
       encryption_key));
-  metadata_buff.disown_data();
 
   return Status::Ok();
 }
@@ -1923,6 +2073,11 @@ Status StorageManager::store_data_to_generic_tile(
       data,
       size);
 
+  return store_data_to_generic_tile(tile, uri, encryption_key);
+}
+
+Status StorageManager::store_data_to_generic_tile(
+    Tile& tile, const URI& uri, const EncryptionKey& encryption_key) {
   GenericTileIO tile_io(this, uri);
   uint64_t nbytes = 0;
   Status st = tile_io.write_generic(&tile, encryption_key, &nbytes);
@@ -1986,10 +2141,6 @@ Status StorageManager::write_to_cache(
   return Status::Ok();
 }
 
-Status StorageManager::write(const URI& uri, Buffer* buffer) const {
-  return vfs_->write(uri, buffer->data(), buffer->size());
-}
-
 Status StorageManager::write(const URI& uri, void* data, uint64_t size) const {
   return vfs_->write(uri, data, size);
 }
@@ -2006,15 +2157,16 @@ tuple<Status, optional<shared_ptr<Group>>> StorageManager::load_group_from_uri(
     const URI& group_uri, const URI& uri, const EncryptionKey& encryption_key) {
   auto timer_se = stats_->start_timer("sm_load_group_from_uri");
 
-  auto&& [st, buffer_opt] = load_data_from_generic_tile(uri, encryption_key);
+  auto&& [st, tile_opt] = load_data_from_generic_tile(uri, 0, encryption_key);
   RETURN_NOT_OK_TUPLE(st, nullopt);
-  auto& buffer = *buffer_opt;
+  auto& tile = *tile_opt;
 
-  stats_->add_counter("read_group_size", buffer.size());
+  stats_->add_counter("read_group_size", tile.size());
 
   // Deserialize
-  ConstBuffer cbuff(&buffer);
-  return Group::deserialize(&cbuff, group_uri, this);
+  Deserializer deserializer(tile.data(), tile.size());
+  auto opt_group = Group::deserialize(deserializer, group_uri, this);
+  return {Status::Ok(), opt_group};
 }
 
 tuple<Status, optional<shared_ptr<Group>>> StorageManager::load_group_details(
@@ -2023,6 +2175,8 @@ tuple<Status, optional<shared_ptr<Group>>> StorageManager::load_group_details(
   auto timer_se = stats_->start_timer("sm_load_group_details");
   const URI& latest_group_uri = group_directory->latest_group_details_uri();
   if (latest_group_uri.is_invalid()) {
+    // Returning ok because not having the latest group details means the group
+    // has just been created and no members have been added yet.
     return {Status::Ok(), std::nullopt};
   }
   return load_group_from_uri(
@@ -2049,6 +2203,8 @@ StorageManager::group_open_for_reads(Group* group) {
     return {Status::Ok(), group_deserialized.value()->members()};
   }
 
+  // Return ok because having no members is acceptable if the group has never
+  // been written to.
   return {Status::Ok(), std::nullopt};
 }
 
@@ -2072,6 +2228,8 @@ StorageManager::group_open_for_writes(Group* group) {
     return {Status::Ok(), group_deserialized.value()->members()};
   }
 
+  // Return ok because having no members is acceptable if the group has never
+  // been written to.
   return {Status::Ok(), std::nullopt};
 }
 
@@ -2094,9 +2252,14 @@ Status StorageManager::load_group_metadata(
   auto status = parallel_for(compute_tp_, 0, metadata_num, [&](size_t m) {
     const auto& uri = group_metadata_to_load[m].uri_;
 
-    auto&& [st, buffer] = load_data_from_generic_tile(uri, encryption_key);
+    auto&& [st, tile] = load_data_from_generic_tile(uri, 0, encryption_key);
     RETURN_NOT_OK(st);
-    metadata_buffs[m] = tdb::make_shared<Buffer>(HERE(), std::move(*buffer));
+
+    metadata_buffs[m] = tdb::make_shared<Buffer>(HERE());
+    RETURN_NOT_OK(metadata_buffs[m]->realloc(tile->size()));
+    metadata_buffs[m]->set_size(tile->size());
+    RETURN_NOT_OK(
+        tile->read(metadata_buffs[m]->data(), 0, metadata_buffs[m]->size()));
 
     return Status::Ok();
   });
@@ -2108,21 +2271,15 @@ Status StorageManager::load_group_metadata(
     meta_size += b->size();
   stats_->add_counter("read_array_meta_size", meta_size);
 
-  // Deserialize metadata buffers
-  auto&& [st, deserialized_metadata] = metadata->deserialize(metadata_buffs);
-  if (!st.ok()) {
-    return st;
-  }
-
   // Copy the deserialized metadata into the original Metadata object
-  *metadata = *(deserialized_metadata.value());
+  *metadata = Metadata::deserialize(metadata_buffs);
   RETURN_NOT_OK(metadata->set_loaded_metadata_uris(group_metadata_to_load));
 
   return Status::Ok();
 }
 
-tuple<Status, optional<Buffer>> StorageManager::load_data_from_generic_tile(
-    const URI& uri, const EncryptionKey& encryption_key) {
+tuple<Status, optional<Tile>> StorageManager::load_data_from_generic_tile(
+    const URI& uri, uint64_t offset, const EncryptionKey& encryption_key) {
   GenericTileIO tile_io(this, uri);
 
   // Get encryption key from config
@@ -2162,16 +2319,17 @@ tuple<Status, optional<Buffer>> StorageManager::load_data_from_generic_tile(
           nullopt);
     }
 
-    auto&& [st1, buff_opt] =
-        tile_io.read_generic(0, encryption_key_cfg, config_);
+    auto&& [st1, tile_opt] =
+        tile_io.read_generic(offset, encryption_key_cfg, config_);
     RETURN_NOT_OK_TUPLE(st1, nullopt);
 
-    return {Status::Ok(), std::move(*buff_opt)};
+    return {Status::Ok(), std::move(*tile_opt)};
   } else {
-    auto&& [st1, buff_opt] = tile_io.read_generic(0, encryption_key, config_);
+    auto&& [st1, tile_opt] =
+        tile_io.read_generic(offset, encryption_key, config_);
     RETURN_NOT_OK_TUPLE(st1, nullopt);
 
-    return {Status::Ok(), std::move(*buff_opt)};
+    return {Status::Ok(), std::move(*tile_opt)};
   }
 
   assert(false);
@@ -2272,11 +2430,17 @@ StorageManager::load_consolidated_fragment_meta(
   if (uri.to_string().empty())
     return {Status::Ok(), nullopt, nullopt};
 
-  auto&& [st, buffer_opt] = load_data_from_generic_tile(uri, enc_key);
+  auto&& [st, tile_opt] = load_data_from_generic_tile(uri, 0, enc_key);
   RETURN_NOT_OK_TUPLE(st, nullopt, nullopt);
-  auto& buffer = *buffer_opt;
+  auto& tile = *tile_opt;
 
-  stats_->add_counter("consolidated_frag_meta_size", buffer.size());
+  stats_->add_counter("consolidated_frag_meta_size", tile.size());
+
+  Buffer buffer;
+  RETURN_NOT_OK_TUPLE(buffer.realloc(tile.size()), nullopt, nullopt);
+  buffer.set_size(tile.size());
+  RETURN_NOT_OK_TUPLE(
+      tile.read(buffer.data(), 0, buffer.size()), nullopt, nullopt);
 
   uint32_t fragment_num;
   buffer.reset_offset();
