@@ -65,6 +65,7 @@ namespace tiledb {
 namespace sm {
 
 class Array;
+class ArrayDirectory;
 class ArraySchema;
 class ArraySchemaEvolution;
 class Buffer;
@@ -78,9 +79,9 @@ class Metadata;
 class OpenArray;
 class MemoryTracker;
 class Query;
+class QueryCondition;
 class RestClient;
 class VFS;
-class ArrayDirectory;
 
 enum class EncryptionType : uint8_t;
 enum class ObjectType : uint8_t;
@@ -149,6 +150,22 @@ class StorageManager {
    */
   Status array_close_for_writes(Array* array);
 
+  /**
+   * Closes an array opened for deletes.
+   *
+   * @param array The array to be closed.
+   * @return Status
+   */
+  Status array_close_for_deletes(Array* array);
+
+  /**
+   * Closes an array opened for updates.
+   *
+   * @param array The array to be closed.
+   * @return Status
+   */
+  Status array_close_for_updates(Array* array);
+
   /**
    * Closes an group opened for reads.
    *
@@ -178,11 +195,12 @@ class StorageManager {
    * Load data from persistent storage.
    *
    * @param uri The object URI.
+   * @param offset The offset into the file to read from.
    * @param encryption_key The encryption key to use.
-   * @return Status, Buffer with the data.
+   * @return Status, Tile with the data.
    */
-  tuple<Status, optional<Buffer>> load_data_from_generic_tile(
-      const URI& uri, const EncryptionKey& encryption_key);
+  tuple<Status, optional<Tile>> load_data_from_generic_tile(
+      const URI& uri, uint64_t offset, const EncryptionKey& encryption_key);
 
   /**
    * Load a group detail from URI
@@ -400,6 +418,26 @@ class StorageManager {
       const std::vector<std::string> fragment_uris,
       const Config* config);
 
+  /**
+   * Writes a commit ignore file.
+   *
+   * @param array_dir The ArrayDirectory where the data is stored.
+   * @param commit_uris_to_ignore The commit files that are to be ignored.
+   */
+  Status write_commit_ignore_file(
+      ArrayDirectory array_dir, const std::vector<URI>& commit_uris_to_ignore);
+
+  /**
+   * Cleans up the array fragments.
+   *
+   * @param array_name The name of the array to be vacuumed.
+   * @param timestamp_start The start timestamp at which to vacuum.
+   * @param timestamp_end The end timestamp at which to vacuum.
+   * @return Status
+   */
+  Status delete_fragments(
+      const char* array_name, uint64_t timestamp_start, uint64_t timestamp_end);
+
   /**
    * Cleans up the array, such as its consolidated fragments and array
    * metadata. Note that this will coarsen the granularity of time traveling
@@ -455,7 +493,7 @@ class StorageManager {
    * @return Status
    */
   Status array_evolve_schema(
-      const ArrayDirectory& array_dir,
+      const URI& array_uri,
       ArraySchemaEvolution* array_schema,
       const EncryptionKey& encryption_key);
 
@@ -469,8 +507,7 @@ class StorageManager {
    *      this instance).
    * @return Status
    */
-  Status array_upgrade_version(
-      const ArrayDirectory& array_dir, const Config* config);
+  Status array_upgrade_version(const URI& array_uri, const Config* config);
 
   /**
    * Retrieves the non-empty domain from an array. This is the union of the
@@ -669,7 +706,7 @@ class StorageManager {
    * @param is_array Set to `true` if the URI is an array and `false` otherwise.
    * @return Status
    */
-  Status is_array(const URI& uri, bool* is_array) const;
+  bool is_array(const URI& uri) const;
 
   /**
    * Checks if the input URI represents a directory.
@@ -767,6 +804,15 @@ class StorageManager {
       const EncryptionKey& encryption_key,
       Metadata* metadata);
 
+  /**
+   * Loads the delete conditions from storage.
+   *
+   * @param array The array.
+   * @return Status, vector of the delete conditions.
+   */
+  tuple<Status, optional<std::vector<QueryCondition>>> load_delete_conditions(
+      const Array& array);
+
   /** Removes a TileDB object (group, array). */
   Status object_remove(const char* path) const;
 
@@ -965,6 +1011,17 @@ class StorageManager {
       const URI& uri,
       const EncryptionKey& encryption_key);
 
+  /**
+   * Stores data into persistent storage.
+   *
+   * @param tile Tile to store.
+   * @param uri The object URI.
+   * @param encryption_key The encryption key to use.
+   * @return Status
+   */
+  Status store_data_to_generic_tile(
+      Tile& tile, const URI& uri, const EncryptionKey& encryption_key);
+
   /** Closes a file, flushing its contents to persistent storage. */
   Status close_file(const URI& uri);
 
@@ -989,15 +1046,6 @@ class StorageManager {
   Status write_to_cache(
       const URI& uri, uint64_t offset, const FilteredBuffer& buffer) const;
 
-  /**
-   * Writes the contents of a buffer into a URI file.
-   *
-   * @param uri The file to write into.
-   * @param buffer The buffer to write.
-   * @return Status.
-   */
-  Status write(const URI& uri, Buffer* buffer) const;
-
   /**
    * Writes the input data into a URI file.
    *
