@@ -36,6 +36,7 @@
 #include <list>
 
 #include <unordered_map>
+#include "tiledb/common/common.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/misc/constants.h"
@@ -171,7 +172,7 @@ class SubarrayPartitioner {
       uint64_t memory_budget_validity,
       ThreadPool* compute_tp,
       stats::Stats* parent_stats,
-      tdb_shared_ptr<Logger> logger);
+      shared_ptr<Logger> logger);
 
   /** Destructor. */
   ~SubarrayPartitioner();
@@ -326,10 +327,10 @@ class SubarrayPartitioner {
   State* state();
 
   /** Returns the subarray. */
-  const Subarray* subarray() const;
+  const Subarray& subarray() const;
 
   /** Returns the subarray. */
-  Subarray* subarray();
+  Subarray& subarray();
 
   /** Returns `stats_`. */
   stats::Stats* stats() const;
@@ -346,7 +347,7 @@ class SubarrayPartitioner {
   inline static std::atomic<uint64_t> logger_id_ = 0;
 
   /** The class logger. */
-  tdb_shared_ptr<Logger> logger_;
+  shared_ptr<Logger> logger_;
 
   /** The config. */
   const Config* config_;
